﻿using System;
using System.Data.SqlClient;
using System.Data;
using System.Transactions;

using Adwk.Mileage.Data;

namespace Adwk.Mileage.Dac
{
    public static class DAHelper1
    {

        // 会員のマイル記録(加算と減算)
        public static bool RecordMileByMember(
            string memberCode,
            DateTime boardingDate, int mile, string cmt,
            out AdwkMStatus stat)
        {
            // [会員マイレージ]を更新し、[マイレージログ]に新規挿入する
            // [会員マイレージ]の[最終更新日]には、データアクセスの日付
            // TransactionScope使用

            SqlConnection conn = new SqlConnection(
                Adwk.Mileage.Dac.Properties.Settings.Default.AdwkMileagesConString);
            SqlCommand select1 = new SqlCommand(); //マイレージ累計の参照
            SqlCommand update1 = new SqlCommand(); //マイレージ累計の更新
            SqlCommand insert1 = new SqlCommand(); //マイレージログの新規挿入

            // パラメータの初期化
            select1.CommandText =
                "SELECT [マイル累計] FROM [会員マイレージ]" +
                " WHERE [会員コード] = @memid";
            select1.Parameters.Add("@memid", SqlDbType.NVarChar);
            //
            update1.CommandText =
                "UPDATE [会員マイレージ]" +
                " SET [マイル累計] = [マイル累計] + @addedValue," +
                "     [最終更新日時] = @udate" +
                " WHERE [会員コード] = @memid";
            update1.Parameters.Add("@addedValue", SqlDbType.Int);
            update1.Parameters.Add("@udate", SqlDbType.DateTime);
            update1.Parameters.Add("@memid", SqlDbType.NVarChar);
            //
            insert1.CommandText =
                "INSERT INTO [マイレージログ](" +
                "[会員コード], [記録日時], [実績マイル], [コメント])" +
                " VALUES(@memid, @rdate, @mile, @cmt)";
            insert1.Parameters.Add("@memid", SqlDbType.NVarChar);
            insert1.Parameters.Add("@rdate", SqlDbType.DateTime);
            insert1.Parameters.Add("@mile", SqlDbType.Int);
            insert1.Parameters.Add("@cmt", SqlDbType.NVarChar);

            try
            {
                using (TransactionScope ts =
                    new TransactionScope(TransactionScopeOption.Required))
                {
                    conn.Open();
                    select1.Connection = conn;
                    update1.Connection = conn;
                    insert1.Connection = conn;

                    // (減算時のみ)現在のマイル残高をチェック
                    if (mile < 0) //減算時のマイル残高の確認
                    {
                        select1.Parameters["@memid"].Value = memberCode;
                        using (SqlDataReader reader1 = select1.ExecuteReader())
                        {
                            if (!reader1.Read()) //該当する会員があるか確認
                            {
                                stat = AdwkMStatus.MemberNotFound;
                                return false;
                            }
                            int totalMile = (int)reader1["マイル累計"];
                            if (totalMile < Math.Abs(mile))
                            {
                                stat = AdwkMStatus.MileNotEnough;
                                return false;
                            }
                        }
                    }

                    //マイル累計の更新
                    update1.Parameters["@addedValue"].Value = mile;     //加算分(減算分)
                    update1.Parameters["@udate"].Value = DateTime.Now;  //最終更新日時
                    update1.Parameters["@memid"].Value = memberCode;    //会員コード
                    int count = update1.ExecuteNonQuery();
                    if (count == 0) //加算の場合は、ここで初めて該当会員の有無チェック
                    {
                        stat = AdwkMStatus.MemberNotFound;
                        return false;
                    }

                    //ログの追加
                    insert1.Parameters["@memid"].Value = memberCode;   //会員コード
                    insert1.Parameters["@rdate"].Value = boardingDate; //記録日時(乗船や消費などの日、計上日)
                    insert1.Parameters["@mile"].Value  = mile;         //マイル
                    insert1.Parameters["@cmt"].Value   = cmt;          //コメント(摘要欄)
                    insert1.ExecuteNonQuery();

                    //正常終了に投票(vote)
                    ts.Complete();
                }

                stat = AdwkMStatus.Success;
                return true;
            }
            catch (SqlException)
            {
                stat = AdwkMStatus.DBError;
            }
            catch (Exception)
            {
                stat = AdwkMStatus.UndefinedError;
            }
            finally
            {
                conn.Close();
            }

            return false;
        }

    }
}
