﻿using System;
using System.ServiceModel;

using Adwk.Mileage.Data;

namespace Adwk.Mileage
{
    /* マイル加算用のインターフェイス
         予約管理サービスの予約データとマイル加算を連携させる必要から、
       分散トランザクションを使用しています。
         このインターフェイスでのやり取りは社内間のアプリケーションに使用されるので、
       カスタムデータ型 AdwkMStatusも使用しています。
    */
    [ServiceContract(Namespace="http://Adwk.Mileage")]
    public interface IMileageUpdate
    {
        // マイルの加算
        [OperationContract]
        [TransactionFlow(TransactionFlowOption.Allowed)]
        bool AddMileByMember(
            string memberCode,         // [in]  会員コード
            DateTime boardingDate,     // [in]  記録日時(乗船や搭乗した日時)
            int baseMile,              // [in]  運行の基本マイル
            string mileageLevel,       // [in]  加算レベル "2"(二等), "1"(一等) など
            string cmt,                // [in]  摘要(乗船経路などを指定)
            out int appliedMile,       // [out] 実際に加算したマイル
            out AdwkMStatus stat,      // [out] ステータス
            out string systemMessage); // [out] システムメッセージ
    }
}
