﻿using System;
using System.ServiceModel;

using Adwk.Mileage.Logic;
using Adwk.Mileage.Data;

namespace Adwk.Mileage.Service
{
    // マイレージ管理サービス ホスト
    class Program
    {
        static void Main(string[] args)
        {
            //TestCode();

            var srv1 = new ServiceHost(typeof(MileageQueryService));
            var srv2 = new ServiceHost(typeof(MileageUpdateService));
            var srv3 = new ServiceHost(typeof(MileageTransferService));

            srv1.Open();
            srv2.Open();
            srv3.Open();

            ShowService(srv1);
            ShowService(srv2);
            ShowService(srv3);

            Console.WriteLine("終了には、何かキーを押してください。");
            Console.ReadKey();

            srv1.Close();
            srv2.Close();
            srv3.Close();
        }

        static void ShowService(ServiceHost srv)
        {
            Console.WriteLine("**** {0} サービス ****", srv.Description.Name);
            //foreach (var hostAddr in srv.BaseAddresses)
            //{
            //    Console.WriteLine("  BaseAddress:{0}", hostAddr);
            //}
            foreach (var endpoint in srv.Description.Endpoints)
            {
                Console.WriteLine("     Endpoint:{0}", endpoint.Address);
            }
            Console.WriteLine();
        }

        // テストコード
        static void TestCode()
        {
            // 各メソッドの基本的な引数と戻り値
            AdwkMStatus stat;
            string msg;
            bool result;

            // マイル加算
            int appliedMile;
            result = BLComp1.AddMileByMember(
                        "0000100001", new DateTime(2011, 6, 11), 100, "2", "定期航路1",
                        out appliedMile, out stat, out msg);
            if (result) Console.WriteLine(stat.ToString());
            else        Console.WriteLine(stat.ToString() + " " + msg);

            result = BLComp1.AddMileByMember(
                        "0000100001", new DateTime(2011, 6, 12), 250, "1", "定期航路2",
                        out appliedMile, out stat, out msg);
            if (result) Console.WriteLine(stat.ToString());
            else Console.WriteLine(stat.ToString() + " " + msg);

            result = BLComp1.AddMileByMember(
                        "0000100003", new DateTime(2011, 6, 13), 250, "1", "定期航路2",
                        out appliedMile, out stat, out msg);
            if (result) Console.WriteLine(stat.ToString());
            else Console.WriteLine(stat.ToString() + " " + msg);

            result = BLComp1.AddMileByMember(
                        null, new DateTime(2011, 6, 13), 250, "1", "定期航路2",
                        out appliedMile, out stat, out msg);
            if (result) Console.WriteLine(stat.ToString());
            else Console.WriteLine(stat.ToString() + " " + msg);

            result = BLComp1.AddMileByMember(
                        "0000100001", new DateTime(2011, 6, 13), 0, "1", "定期航路2",
                        out appliedMile, out stat, out msg);
            if (result) Console.WriteLine(stat.ToString());
            else Console.WriteLine(stat.ToString() + " " + msg);

            result = BLComp1.AddMileByMember(
                        "0000100001", new DateTime(2011, 6, 13), 250, null, "定期航路2",
                        out appliedMile, out stat, out msg);
            if (result) Console.WriteLine(stat.ToString());
            else Console.WriteLine(stat.ToString() + " " + msg);

            // 会員情報、マイル累計
            var table1 = new AdwkMileagesDataSet.会員マイル累計DataTable();

            result = BLComp1.QueryInfo1ByMember("0000100001", table1, out stat, out msg);
            if (result) Console.WriteLine("member={0}{1}", table1[0].姓, table1[0].名);
            else        Console.WriteLine(msg);

            result = BLComp1.QueryInfo1ByMember("0000100003", table1, out stat, out msg);
            if (result) Console.WriteLine("member={0}{1}", table1[0].姓, table1[0].名);
            else Console.WriteLine(msg);

            // マイル履歴
            var table2 = new AdwkMileagesDataSet.会員マイル履歴DataTable();
            result = BLComp1.QueryHistoryByMemberAndPeriod(
                        "0000100001", new DateTime(2011, 06, 01), new DateTime(2011, 07, 01),
                        table2, out stat, out msg);
            if (result)
            {
                foreach (AdwkMileagesDataSet.会員マイル履歴Row row in table2)
                {
                    Console.WriteLine("{0} {1} {2} {3:yy/MM/dd} {4:#,##0} {5}", 
                        row.会員コード, row.姓, row.名,
                        row.記録日時, row.実績マイル, row.コメント);
                }
            }
            else
            {
                Console.WriteLine(msg);
            }
        }
    }
}

