﻿using System;
using System.Transactions;


namespace DummyExternalService
{
    public static class PointBLComp1
    {
        // このコンポーネントが返すメッセージ
        const string msgCustomerNotFound = "指定されたID、および氏名に該当する方は存在しません。";
        const string msgPointInvalid = "移行ポイントの値が不正です(1以上でなければなりません)。";
        const string msgDbError = "システムエラーのため、受付できません。(Error 001)";
        const string msgEtcError = "システムエラーのため、受付できません。(Error 002)";
        const string msgSuccess  = "ポイントの移行を受付ました。";

        // 加算すべきポイントの受付け
        //   WCFサービスに依存しないビジネスロジックのコンポーネント
        //   データアクセスコンポーネントを呼び出して、必要に応じてメッセージを返す
        public static bool TransferPoint(
            string fromSystem,
            string toCustomerId, string lastName, string firstName,
            int point, out int transferId, out string message)
        {

            const string cmtTransfer = "{0}からポイント移行";
            
            bool result;
            string cmt;
            PointStatus stat;

            using (TransactionScope ts =
                    new TransactionScope(TransactionScopeOption.Required))
            {
                //移行ポイントの検証
                if (point <= 0)
                {
                    transferId = 0; //dummy
                    message = msgPointInvalid;
                    return false;
                }

                //摘要作成
                cmt = String.Format(cmtTransfer, fromSystem);

                //データベースへの移行データ追加
                result = PointDAHelper1.TransferPoint(
                            toCustomerId, lastName, firstName,
                            point, cmt, out transferId, out stat);

                if (!result)
                {
                    switch (stat)
                    {
                        case PointStatus.CustomerNotFound:
                            message = msgCustomerNotFound;
                            break;
                        case PointStatus.DBError:
                            message = msgDbError;
                            break;
                        case PointStatus.EtcError:
                        default:
                            message = msgEtcError;
                            break;
                    }
                    //データ追加できない場合はここで終了
                    return false;
                }

                //正常終了メッセージ
                message = msgSuccess;

                //正常終了に投票(vote)
                ts.Complete();
            }

            return true;
        }
    }
}
