﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.Data.SqlClient;
using System.Data;
using System.Transactions;

namespace DummyExternalService
{
    //ポイント受付に関するステータス
    public enum PointStatus
    {
        Success,
        CustomerNotFound,
        DBError,
        EtcError
    }

    public static class PointDAHelper1
    {
        // 加算すべきポイントの受付けのためのデータベース更新
        public static bool TransferPoint(
            string toCustomerId, string lastName, string firstName,
            int point, string cmt, out int transferId, out PointStatus stat)
        {
            // [Customers]を照会して確認し、[RequestLogs]に新規挿入する
            // [RequestLogs]の[RequestDate]には、データ更新の日付
            // TransactionScope使用

            SqlConnection conn = new SqlConnection(
                        DummyExternalService.Properties.Settings.Default.ExternalPointConstring);
            SqlCommand select1 = new SqlCommand(); // [Customers] の参照
            SqlCommand insert1 = new SqlCommand(); // [RequestLogs] への新規挿入
            SqlCommand select2 = new SqlCommand(); // 新規挿入時の [RequestId] (移行Id)の取得

            // コマンドの設定とパラメータの初期化
            select1.CommandText =
                "SELECT [LastName], [FirstName] FROM [Customers]" +
                " WHERE [CustomerId] = @custid";
            select1.Parameters.Add("@custid", SqlDbType.NVarChar);
            //
            insert1.CommandText =
                "INSERT INTO [TransferRequests](" +
                "[CustomerId], [RequestDate], [RequestPoint], [Comment])" +
                " VALUES(@custid, @rdate, @rpoint, @cmt)";
            insert1.Parameters.Add("@custid", SqlDbType.NVarChar);
            insert1.Parameters.Add("@rdate" , SqlDbType.DateTime);
            insert1.Parameters.Add("@rpoint", SqlDbType.Int);
            insert1.Parameters.Add("@cmt"   , SqlDbType.NVarChar);
            //
            select2.CommandText = "SELECT @@IDENTITY";

            try
            {
                using (TransactionScope ts =
                    new TransactionScope(TransactionScopeOption.Required))
                {
                    conn.Open();
                    select1.Connection = conn;
                    insert1.Connection = conn;
                    select2.Connection = conn;

                    // 指定された顧客コード(toCustomerId)、名義(lastName, firstName)が
                    // 実在する情報と一致するか検証
                    // 一致しない場合は、移行を受け付けない
                    select1.Parameters["@custId"].Value = toCustomerId;
                    SqlDataReader reader1 = select1.ExecuteReader();
                    if (!reader1.Read() ||
                        (reader1["LastName"].ToString()  != lastName) ||
                        (reader1["FirstName"].ToString() != firstName))
                    {
                        reader1.Close();
                        transferId = 0; //dummy
                        stat = PointStatus.CustomerNotFound;
                        return false;
                    }
                    reader1.Close();

                    //移行データの新規作成
                    insert1.Parameters["@custid"].Value = toCustomerId;  //CustomerId
                    insert1.Parameters["@rdate"].Value  = DateTime.Now;  //RequestDate
                    insert1.Parameters["@rpoint"].Value = point;         //RequestPoit
                    insert1.Parameters["@cmt"].Value = cmt;              //Comment
                    insert1.ExecuteNonQuery();                    

                    //新規挿入時の移行Id(TransferRequestId)の取得
                    transferId = Convert.ToInt32(select2.ExecuteScalar());

                    //正常終了に投票(vote)
                    ts.Complete();
                }

                stat = PointStatus.Success;
                return true;
            }
            catch (SqlException)
            {
                transferId = 0;  //dummy
                stat = PointStatus.DBError;
            }
            catch (Exception)
            {
                transferId = 0;  //dummy
                stat = PointStatus.EtcError;
            }
            finally
            {
                conn.Close();
            }

            return false;
        }
    }
}
