﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Adwk.Reservation.Dac;

namespace Adwk.Reservation.Logic
{
    // 予約管理サービスの基本情報を返すサービス
    //   特にステートを維持する必要はないので、
    // 静的クラスとして実装しています。
    public static class BLComp1
    {
        // 場所(出発地、到着地)、クラス(一等、二等)の基本情報を取得する
        //   取得に成功した場合に戻り値はtrue
        //   エンティティセットを返すバージョン
        public static bool GetBasicRouteInfo(
            out IEnumerable<場所> places, out IEnumerable<クラス> levels)
        {
            places = null;
            levels = null;

            try
            {
                // 返すべき2つのテーブルは、大きくないことを前提に
                // ObejectContext の 2つのエンティティセットを返す
                var oc = new AdwkReservationsEntities();

                // ナビゲーションプロパティは必要ないので、
                // 参照した際に読み込まないようにする
                oc.ContextOptions.LazyLoadingEnabled = false;
                //oc.ContextOptions.LazyLoadingEnabled = true;

                // 2つのエンティティセットの出力引数に設定
                // 遅延実行のため、実際にデータが取得されるのは、
                // WCFサービスがシリアル化して返すとき
                places = oc.場所;
                levels = oc.クラス;

                // 遅延実行のため、ここでのリソース破棄は不可!
                // oc.Dispose();
            }
            catch
            {
                return false;
            }

            return true;
        }

        // 場所(出発地、到着地)、クラス(一等、二等)の基本情報を取得する
        //   取得に成功した場合に戻り値はtrue
        //   エンティティの配列を返すバージョン
        public static bool GetBasicRouteInfo(
            out 場所 [] places, out クラス [] levels)
        {
            places = null;
            levels = null;

            try
            {
                // 返すべき2つのテーブルは、大きくないことを前提に
                // ObejectContext の 2つのエンティティセットを返す
                var oc = new AdwkReservationsEntities();

                // ナビゲーションプロパティは必要ないので、
                // 参照した際に読み込まないようにする
                oc.ContextOptions.LazyLoadingEnabled = false;
                //oc.ContextOptions.LazyLoadingEnabled = true;

                // 配列を作成することで、この時点でクエリを実行させる
                places = oc.場所.ToArray();
                levels = oc.クラス.ToArray();

                // クエリは実行済みなのでリソースを破棄できる
                oc.Dispose();
            }
            catch
            {
                return false;
            }

            return true;
        }

        // 場所(出発地、到着地)、クラス(一等、二等)の基本情報を取得する
        //   取得に成功した場合に戻り値はtrue
        //   カスタムデータ型の配列を返すバージョン
        public static bool GetBasicRouteInfo(
            out PlaceInfo[] places, out LevelInfo[] levels)
        {
            places = null;
            levels = null;

            try
            {
                // 返すべき2つのテーブルは、大きくないことを前提に
                // ObejectContext の 2つのエンティティセットを返す
                var oc = new AdwkReservationsEntities();

                // ナビゲーションプロパティは必要ないので、
                // 参照した際に読み込まないようにする
                oc.ContextOptions.LazyLoadingEnabled = false;
                //oc.ContextOptions.LazyLoadingEnabled = true;

                // カスタムデータ型のためにクエリ作成
                var query1 = from p in oc.場所
                             select new PlaceInfo
                             {
                                 PlaceCode = p.場所コード,
                                 PlaceName = p.場所名
                             };
                var query2 = from p in oc.クラス
                             select new LevelInfo
                             {
                                 LevelCode = p.クラスコード,
                                 LevelName = p.クラス名
                             };
                // 遅延実行のため、実際にデータが取得されるのは、

                // 配列を作成することで、この時点でクエリを実行させる
                places = query1.ToArray();
                levels = query2.ToArray();

                // クエリは実行済みなのでリソースを破棄できる
                oc.Dispose();
            }
            catch
            {
                return false;
            }

            return true;
        }
    }
}
