﻿using System;
using System.Linq;
using System.Collections.Generic;

using System.ServiceModel;

using Adwk.Reservation.Dac;


namespace Adwk.Reservation
{
    // 予約管理サービスのインターフェイス
    // (予定照会、予約、予約照会、予約削除)
    [ServiceContract(
        Namespace="http://Adwk.Reservation",
        SessionMode=SessionMode.Required)]
    public interface IReservation
    {

        // 運行予定の取得
        //   簡単にするため、引数 deptCode, arrvCode は必須
        //   引数level は "" や null を指定可能(ワイルドカード)
        [OperationContract]
        bool QuerySchedule(
            DateTime fromDate, DateTime untilDate,
            string deptCode, string arrvCode, string levelCode,
            out ScheduleInfo[] sinfos,
            out string systemMessage);

        // 予約処理
        //   運行コード scheduleCode とクラスコード levelCode を指定することで、予約対象を特定できます。
        //   簡単にするため、必要な利用者(乗客)の情報は氏名だけです。
        //   会員コード memberCode は、"" や null も可能。
        // 簡単にするため以下のように実装してあります。
        //   この時点では会員コードの有効性は検証しません。
        //   (会員コードは、別のマイレージ管理サービスの管理下にあります。)
        //   (乗船手続きの際に、マイル加算するので、その際に有効性をチェックします。)
        //   引数 appliedPrice と appliedTax は、最終的に予約時に決定される適用料金であり、
        //   呼び出し側が自由に設定できますが、このサンプルでは、その有効性を検証しません。
        //   引数 cmt は、予約時に呼び出し元が任意に追加するコメントです。
        //   (コメントとして書き込める内容に制限はありません。)
        [OperationContract]
        bool Reserve(
            string scheduleCode, string levelCode,
            string memberCode, string lastName, string firstName,
            decimal appliedPrice, decimal appliedTax, string cmt,
            out string reservationCode, out string systemMessage);

        // 予約の照会(ReservationInfoとして返すバージョン)
        //   指定した予約コード(reservationCode)、氏名(lastnameとfirstName)が
        // 一致した場合のみ、予約データを返します。
        [OperationContract]
        bool QueryReservation1ByRCode(
            string reservationCode, string lastName, string firstName,
            out ReservationInfo reservation,
            out string systemMessage);

        // 予約の削除
        //   ここでは「照会したのちに削除する」という手順を呼び出し側が行うという
        // 前提なので、削除を指定した際には、そのデータが現在照会済みであるかを
        // チェックした後に削除しています。
        [OperationContract]
        bool DeleteReservationByRCode(
            string reservationCode,
            out string systemMessage);

        // サービス側で確保されたステートを強制的にリセット
        [OperationContract]
        void Reset();

        // (テスト用メソッド)サービスインスタンスのハッシュ値を調べる
        [OperationContract]
        int GetInstanceHashCode();
    }
}
