﻿using System;
using System.Threading;

using Adwk.Reservation.Data;
using Adwk.Reservation.Logic;

namespace Adwk.Reservation.Service
{
    // 予約管理サービスのインターフェイス
    // (乗船関連の処理)
    public class RBoardingService
        : IRBoarding
    {
        // このサービス固有のメッセージ
        const string msgAccessError =
            "乗船手続きのサービスを利用する権限がありません。";
        const string msgGeneralError =
            "(内部エラー) システムメッセージ : {0}";

        // アクセス可能なロール
        const string validRole1 = @"builtin\users";

        // 乗船の手続き
        //   指定された運行コード(scheduleCode)とクラス(levelCode)に対して
        //   指定された予約コード(reservationCode)の乗客(lastName、firstName)に関する
        //   チェックイン(乗船手続き)を行います。
        //   (乗船にはあらかじめ予約が必要です。)
        //   指定された情報に一致する予約がない場合、手続きは失敗します。
        //   この時点で会員コードを指定することで、既存の予約データの会員コードを上書きすることができます。
        bool IRBoarding.CheckIn(
            string scheduleCode, string levelCode,
            string reservationCode, string memberCode, 
            string lastName, string firstName,
            out string systemMessage)
        {
            // ロールを用いたアクセス権の確認
            var principal1 = Thread.CurrentPrincipal;
            Console.WriteLine("[ReservationService.CheckIn]");
            Console.WriteLine(
                "  Type : {0}", principal1.Identity.AuthenticationType);
            Console.WriteLine(
                "  User : {0}", principal1.Identity.Name);
            Console.WriteLine("  {0} ? : {1}", 
                validRole1, (principal1.IsInRole(validRole1)) ? "OK" : "NG" );
            if (! Thread.CurrentPrincipal.IsInRole(validRole1))
            {
                systemMessage = msgAccessError;
                return false;
            }
            //
            
            bool result;
            string msg;
            AdwkRStatus stat;
            
            // 乗船手続きの呼び出し
            result = BLComp4.CheckIn(
                scheduleCode, levelCode,
                reservationCode, memberCode, lastName, firstName,
                out stat, out msg);

            // statは返さない
            // 返すメッセージの補正
            if (result) systemMessage = msg;
            else systemMessage = String.Format(msgGeneralError, msg);

            return result;
        }
    }
}
