﻿using System;
using System.Windows.Forms;

using System.ServiceModel;
using System.ServiceModel.Discovery;
using System.ServiceModel.Routing;
using System.ServiceModel.Security;

// プロキシの名前空間
using DummyExternalApp.ServiceAdwkM;
using DummyExternalApp.ServiceAdwkR;

namespace DummyExternalApp
{
    public partial class Form1 : Form
    {
        // サービスを呼び出す際のエンドポイントアドレス
        EndpointAddress epaR = null;
        EndpointAddress epaM = null;

        WSHttpBinding custBinding = new WSHttpBinding();

        // 各サービスを識別するホスト名やポートを除くURI(絶対パス)
        const string mserviceURI = "/adwkrouting/adwkm";
        const string rserviceURI = "/adwkrouting/adwkr";

        // offline時のアドレス欄表示
        const string offlineDisplay = "(offline)";

        // サービス利用可能フラグ(UI更新などで使用)
        private bool isMServiceAvailable = false;
        private bool isRServiceAvailable = false;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            custBinding.Security.Mode = SecurityMode.Message;
            custBinding.Security.Message.ClientCredentialType =
                MessageCredentialType.UserName;

            FindService(); //探索
            SetupListen(); //アナウンスメント

            //UIスレッドチェック
            //MessageBox.Show("Thread=" +
            //    System.Threading.Thread.CurrentThread.ManagedThreadId);
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            CloseListen();
        }

        // 接続状態に応じてボタンなどの UI更新
        private void UpdateUI()
        {
            btnMileageQuery.Enabled = isMServiceAvailable;
            btnSchQuery.Enabled = isRServiceAvailable;
            btnResQuery.Enabled = isRServiceAvailable;
            btnResAdd.Enabled   = isRServiceAvailable;
            btnResDelete.Enabled = isRServiceAvailable;
        }
        
        // 予約の照会
        private void btnMileageQuery_Click(object sender, EventArgs e)
        {
            var clientM = new MileageQueryClient(custBinding, epaM);

            try
            {
                clientM.ClientCredentials.UserName.UserName = txtSID.Text;
                clientM.ClientCredentials.UserName.Password = txtSpass.Text;
                clientM.ClientCredentials
                    .ServiceCertificate.Authentication.CertificateValidationMode =
                        X509CertificateValidationMode.PeerOrChainTrust;

                MemberInfo minfo;
                bool resultM = clientM.QueryInfo1ByMember(out minfo, txtMCode.Text);

                clientM.Close();

                MessageBox.Show("  ***  マイレージ照会結果  ***  " +
                    Environment.NewLine + Environment.NewLine +
                    "MemberCode : " + minfo.MemberCode + Environment.NewLine +
                    "LastName : " + minfo.LastName + Environment.NewLine +
                    "FirstName : " + minfo.FirstName + Environment.NewLine +
                    "TotalMile : " + minfo.TotalMile.ToString());

            }
            catch (Exception ex)
            {
                MessageBox.Show(
                    ex.Message + "\n" +
                    (ex.InnerException.Message ?? ""));
                clientM.Abort();
            }
        }

        // 予約サービス用のプロキシ
        // 複数のイベントハンドラーをまたいで同一セッションを使用するので、
        // プロキシをメンバー変数として確保
        private ReservationClient clientR = null;

        // 予約照会
        // (予約の削除と同一セッションを使用)
        private void btnResQuery_Click(object sender, EventArgs e)
        {
            try
            {
                // 簡単にするため、その都度、ログインを行う
                clientR = new ReservationClient(custBinding, epaR);
                clientR.ClientCredentials.UserName.UserName = txtSID.Text;
                clientR.ClientCredentials.UserName.Password = txtSpass.Text;
                clientR.ClientCredentials.
                    ServiceCertificate.Authentication.CertificateValidationMode =
                        X509CertificateValidationMode.PeerOrChainTrust;
                //

                ReservationInfo rinfo = new ReservationInfo();
                string msg = "";

                clientR.QueryReservation1ByRCode(out rinfo, out msg, txtResCode1.Text,
                    txtRLName.Text, txtRFName.Text);

                MessageBox.Show("  ***  予約データ照会結果 ***  " + Environment.NewLine + Environment.NewLine +
                  "ReservationCode : " + rinfo.ReservationCode + Environment.NewLine +
                  "MemberCode : " + rinfo.MemberCode + Environment.NewLine +
                  "LastName : " + rinfo.LastName + Environment.NewLine +
                  "FirstName : " + rinfo.FirstName + Environment.NewLine +
                  "ScheduleCode : " + rinfo.ScheduleCode + Environment.NewLine +
                  "ScheduleDate : " + rinfo.ScheduleDate + Environment.NewLine +
                  "LevelCode : " + rinfo.LevelCode + Environment.NewLine +
                  "LevelName : " + rinfo.LevelName + Environment.NewLine +
                  "CruiseId : " + rinfo.CruiseId + Environment.NewLine +
                  "DeptName : " + rinfo.DeptName + Environment.NewLine +
                  "DeptTime : " + rinfo.DeptTime + Environment.NewLine +
                  "ArrvName : " + rinfo.ArrvName + Environment.NewLine +
                  "arrvTime : " + rinfo.ArrvTime + Environment.NewLine +
                  "AppliedPrice : " + rinfo.AppliedPrice.ToString() + Environment.NewLine +
                  "AppliedTax : " + rinfo.AppliedTax.ToString() + Environment.NewLine +
                  "BoardingFlag : " + rinfo.BoardingFlag + Environment.NewLine +
                  "BoardingDate : " + rinfo.BoardingDate.ToString() + Environment.NewLine +
                  "AppliedMile : " + rinfo.AppliedMile.ToString() + Environment.NewLine + Environment.NewLine +
                  "SystemMessage : " + msg);

                txtResCode2.Text = rinfo.ReservationCode;

            }
            catch (Exception ex)
            {
                MessageBox.Show(
                    ex.Message + "\n" +
                    (ex.InnerException.Message ?? ""));
                clientR.Abort();
                clientR = null;
            }

        }

        // 予約追加
        private void btnResAdd_Click(object sender, EventArgs e)
        {
            var clientR = new ReservationClient(custBinding, epaR);
            try
            {
                clientR.ClientCredentials.UserName.UserName = txtSID.Text;
                clientR.ClientCredentials.UserName.Password = txtSpass.Text;
                clientR.ClientCredentials
                    .ServiceCertificate.Authentication.CertificateValidationMode =
                        X509CertificateValidationMode.PeerOrChainTrust;

                string msg = "";
                string resCode = "";

                clientR.Reserve(out resCode, out msg, txtSchduleCode.Text,txtLevelC.Text,
                    txtMemCode.Text,txtLName.Text,txtFName.Text,Convert.ToDecimal(txtApPrice.Text),
                    Convert.ToDecimal(txtTax.Text),txtComm.Text);

                clientR.Close();

                MessageBox.Show("  ***  予約データ登録結果 ***  " + Environment.NewLine + Environment.NewLine +
                  "ReservationId : " + resCode + Environment.NewLine + Environment.NewLine +
                  "SystemMessage : " + msg);

            }
            catch (Exception ex)
            {
                MessageBox.Show(
                    ex.Message + "\n" +
                    (ex.InnerException.Message ?? ""));
                clientR.Abort();
                clientR = null;
            }
        }

        // 予約削除
        // (予約の照会と同一のセッションを使用)
        private void btnResDelete_Click(object sender, EventArgs e)
        {
            try
            {
                if (clientR == null)
                {
                    return;
                }

                string msg = "";

                clientR.DeleteReservationByRCode(out msg, txtResCode2.Text);
              
                MessageBox.Show("  ***  予約データ削除結果 ***  " + Environment.NewLine + Environment.NewLine + 
                    "SystemMessage : " + msg);

            }
            catch (Exception ex)
            {
                MessageBox.Show(
                    ex.Message + "\n" +
                    (ex.InnerException.Message ?? ""));
                clientR.Abort();
                clientR = null;
            }
        }

        private void btnSchQuery_Click(object sender, EventArgs e)
        {

        }

        //対象となるサービスを探索するメソッド
        // (アプリケーション起動時に実行して正しいURLを取得すること)
        void FindService()
        {
            // wsHttpBinding を使用するDiscoveryClient の場合
            var endpoint1 = new DiscoveryEndpoint();
            endpoint1.Address =
                new EndpointAddress("http://localhost:8090/adwkrouting/mydisco");
            endpoint1.Binding = new WSHttpBinding();
            var discoveryClient = new DiscoveryClient(endpoint1);

            // UDP を使用する DiscoveryClient の場合
            //var discoveryClient = new DiscoveryClient(new UdpDiscoveryEndpoint());

            // サービスの探索
            FindResponse routingServices = null;
            try
            {
                routingServices = discoveryClient.Find(
                    new FindCriteria(typeof(IRequestReplyRouter)));
            }
            catch
            {
                // wsHttpBindingでは、検索サービスが見つからないと例外が発生する
                // UDPでは、検索サービスが見つからないと、正常に呼び出し終了し、
                // Endpoints.Countはゼロになる
            }

            // サービス参照で取得した特定のインターフェイスを検索する場合
            //FindCriteria criteria = new FindCriteria();
            //XmlQualifiedName name = new XmlQualifiedName("ICalc", "http://tempuri.org/");
            //criteria.ContractTypeNames.Add(name);

            if (routingServices == null ||
                routingServices.Endpoints.Count == 0)
            {
                //MessageBox.Show("*** サービスが見つかりませんでした ***");
                labelMAddress.Text = offlineDisplay;
                labelRAddress.Text = offlineDisplay;
                UpdateUI(); // ボタンなどの UI更新
                return;
            }
            else
            {
                //MessageBox.Show(
                //    String.Format("*** {0}個のサービスが見つかりました ***",
                //                  routingServices.Endpoints.Count));
                 foreach (EndpointDiscoveryMetadata endpoint
                    in routingServices.Endpoints)
                {
                    switch (endpoint.ListenUris[0].AbsolutePath)
                    {
                        case mserviceURI:
                            epaM = endpoint.Address; //マイレージサービスの絶対アドレス
                            labelMAddress.Text = epaM.ToString();
                            isMServiceAvailable = true;
                            break;
                        case rserviceURI:
                            epaR = endpoint.Address; //予約サービスの絶対アドレス
                            labelRAddress.Text = epaR.ToString();
                            isRServiceAvailable = true;
                            break;
                        default:
                            break;
                    }
                }

                UpdateUI(); // ボタンなどの UI更新
            }
        }

        // アナウンスメントのリッスン用オブジェクト
        private ServiceHost announcementServiceHost;
        private AnnouncementService announcementService;

        // アナウンスメントのリッスンの準備
        private void SetupListen()
        {
            // AnnouncementService のインスタンス作成
            announcementService = new AnnouncementService();

            // イベントハンドラーの設定
            announcementService.OnlineAnnouncementReceived += OnOnline;
            announcementService.OfflineAnnouncementReceived += OnOffline;

            // AnnouncementService の実行
            announcementServiceHost = new ServiceHost(announcementService);
            announcementServiceHost.AddServiceEndpoint(
                new UdpAnnouncementEndpoint());
            announcementServiceHost.Open();
        }

        // アナウンスメントのリッスンの終了
        private void CloseListen()
        {
            if(announcementServiceHost != null)
                announcementServiceHost.Close();
        }

        // サービスがオンラインになった際
        // (エンドポイントの数だけ呼び出される)
        private void OnOnline(object sender, AnnouncementEventArgs e)
        {
            // UIスレッドでイベントを受け取ることができる
            //MessageBox.Show(
            //    String.Format(
            //        "Received an online announcement from {0}, Thread={1}",
            //        e.EndpointDiscoveryMetadata.Address,
            //        System.Threading.Thread.CurrentThread.ManagedThreadId));

            switch (e.EndpointDiscoveryMetadata.Address.Uri.AbsolutePath)
            {
                case mserviceURI:
                    epaM = e.EndpointDiscoveryMetadata.Address; //マイレージサービスの絶対アドレス
                    labelMAddress.Text = epaM.ToString();
                    isMServiceAvailable = true;
                    break;
                case rserviceURI:
                    epaR = e.EndpointDiscoveryMetadata.Address; //予約サービスの絶対アドレス
                    labelRAddress.Text = epaR.ToString();
                    isRServiceAvailable = true;
                    break;
                default:
                    break;
            }

            // ボタンなどの UI更新
            UpdateUI();
        }

        // サービスがオフラインになった際
        // (エンドポイントの数だけ呼び出される)
        private void OnOffline(object sender, AnnouncementEventArgs e)
        {
            // UIスレッドでイベントを受け取ることができる
            //MessageBox.Show(
            //    String.Format(
            //        "Received an offline announcement from {0}, Thread={1}",
            //        e.EndpointDiscoveryMetadata.Address,
            //        System.Threading.Thread.CurrentThread.ManagedThreadId));

            switch (e.EndpointDiscoveryMetadata.Address.Uri.AbsolutePath)
            {
                case mserviceURI:
                    epaM = null;
                    labelMAddress.Text = offlineDisplay;
                    isMServiceAvailable = false;
                    break;
                case rserviceURI:
                    epaR = null;
                    labelRAddress.Text = offlineDisplay;
                    isRServiceAvailable = false;
                    break;
                default:
                    break;
            }

            // ボタンなどの UI更新
            UpdateUI();
        }
    }
}
