﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.Security;

using Adwk.WebApp.ServiceReference1;

namespace Adwk.WebApp
{
    public partial class MasterPage : System.Web.UI.MasterPage
    {
        // 使用するセッションデータ
        //   Session[SVars.UserName] ---- ページ上部に表示するユーザー名
        //   Session[SVars.MemberInfo] ---- ユーザー情報(SVar.UserNameと同期)

        protected void Page_Load(object sender, EventArgs e)
        {
            string username = "";
            // string rolename;  // ユーザーによってロール別管理を行う際のロール名退避用変数

            if (Page.User.Identity.IsAuthenticated)
            {
                //ログインの状況に応じて表示内容を切り替え
                LoginPanel.Visible = false;
                LogoutPanel.Visible = true;
                //ユーザー名の表示
                if(Session[SVars.UserName] != null)
                {
                    username = Session[SVars.UserName] as string;
                }
                else
                {
                    // マイル管理サービスを使用して、
                    // Webのログインユーザー名(会員コード)から会員のユーザー名を取得する
                    var proxy = new MileageQueryClient();

                    bool result;
                    MemberInfo minfo = null;

                    result = proxy.QueryInfo1ByMember(
                        out minfo, Page.User.Identity.Name);

                    proxy.Close();

                    if (result)
                    {
                        username = minfo.LastName + " " + minfo.FirstName;
                        Session[SVars.UserName] = username; //2回目以降は、WCFサービスアクセスを避けるため、ユーザー名を退避
                        Session[SVars.MemberInfo] = minfo;  //MemberInfoは「フェリーの予約」/「マイル確認と使用」のページでも使用
                    }
                    else
                    {
                        // Webのログイン ユーザー名(会員コード)に対応する
                        // 会員ユーザー名はあるはずなので、
                        // 本来はここは実行されない。
                        username = "???";
                        Session[SVars.UserName] = null;
                        Session[SVars.MemberInfo] = null;
                    }
                 }
                 //ロールによる表示切替などがある場合
                 //if Page.User.IsInRole("Gold"){
                 //    rolename = "(優待会員様)";
                 //else
                 //{
                 //    rolename = "":
                 //}               
            }
            else
            {
                //ログインの状況に応じて表示内容を切り替え
                LoginPanel.Visible = true;
                LogoutPanel.Visible = false;
                
                username = "ゲスト";
                //rolename = "";
    
                //念のため、未認証の場合はセッション変数をクリア
                Session[SVars.UserName] = null;
                Session[SVars.MemberInfo] = null;
            }
            Msg1.Text = String.Format(
                 "ようこそ、{0} さん", username ?? "");
            
            // ロールがある場合
            //Msg1.Text = String.Format( 
            //    "ようこそ、{0} さん{1}", username ?? "", rolename ?? ""); 
        }

        protected void LoginButton_Click(object sender, EventArgs e)
        {
            Response.Redirect("~/Login.aspx");
        }

        protected void LogoutButton_Click(object sender, EventArgs e)
        {
            FormsAuthentication.SignOut();  //ログアウト
            Session.Abandon();              //サーバー上のセッションデータのクリア
            Response.Redirect("~/Logout.aspx");
        }
    }
}