﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.Text;

using System.ServiceModel.Web;
using Adwk.Reservation;
using System.ServiceModel.Activation;

namespace Adwk.WebApp
{
    // 予約管理サービス
    // (予定照会、予約、予約照会、予約削除)
    [ServiceContract(Namespace = "http://adwk.webapp.reservation")]
    public interface IReservation
    {
        // 運行予定の取得
        //   簡単にするため、引数 deptCode, arrvCode は必須
        //   引数level は "" や null を指定可能(ワイルドカード)
        [OperationContract]
        [WebGet(
            UriTemplate = "QuerySchedule?date={thisDate}&" +
                          "dept={deptCode}&arrv={arrvCode}&" +
                          "level={levelCode}",
            ResponseFormat = WebMessageFormat.Json)]
        ScheduleBasicInfo[] QuerySchedule(
            DateTime thisDate, string deptCode, string arrvCode, string levelCode);

        // 予約処理
        //   運行コード scheduleCode とクラスコード levelCode を指定することで、予約対象を特定できます。
        //   簡単にするため、必要な利用者(乗客)の情報は氏名だけです。
        //   会員コード memberCode は、"" や null も可能。
        // 簡単にするため以下のように実装してあります。
        //   この時点では会員コードの有効性は検証しません。
        //   (会員コードは、別のマイレージ管理サービスの管理下にあります。)
        //   (ここでは簡単にするため、このメソッドの引数として価格は指定できません。)
        [OperationContract]
        [WebInvoke(
            Method="POST",
            UriTemplate="Reservations",
            RequestFormat = WebMessageFormat.Json,
            ResponseFormat = WebMessageFormat.Json)]
        string ReserveByWeb(ReservationRuqestInfo reqInfo);

        // 予約の照会(ReservationBasicInfoとして返すバージョン)
        //   指定した予約コード(reservationCode)の予約データを返します。
        //   使用しているセッションのユーザー名が、予約データに一致している
        //   必要があります。
        [OperationContract]
        [WebGet(
            UriTemplate = "Reservations?rcode={reservationCode}",
            ResponseFormat = WebMessageFormat.Json)]
        ReservationBasicInfo QueryReservation1ByRCode(string reservationCode);

        // 予約の削除
        //   ここでは「照会したのちに削除する」という手順を呼び出し側が行うという
        // 前提なので、削除を指定した際には、そのデータが現在照会済みであるかを
        // チェックした後に削除しています。
        [OperationContract]
        [WebInvoke(
            Method="DELETE",
            UriTemplate="Reservations?rcode={reservationCode}",
            ResponseFormat = WebMessageFormat.Json)]
        string DeleteReservationByRCode(string reservationCode);

        // サービス側で確保されたステートを強制的にリセット
        //[OperationContract]
        //void Reset();

        // (テスト用メソッド)サービスインスタンスのハッシュ値を調べる
        [OperationContract]
        [WebGet(
            UriTemplate = "GetInstanceHashCode",
            ResponseFormat = WebMessageFormat.Json)]
        int GetInstanceHashCode();
    }

    //Webページ表示用の最小限の運行データ
    //(ここではサンプルのWCFサービス確認のため、やり取りすべきデータを最小限にしています)
    [DataContract]
    public class ScheduleBasicInfo   // 1便分のスケジュール
    {
        [DataMember] public string ScheduleCode;    // 運行コード(予約のときに必要)
        [DataMember] public string ScheduledDate;   // 運航日
        [DataMember] public string CruiseId;        // 便名
        [DataMember] public string DeptName;  // 出発地名
        [DataMember] public string DeptTime;  // 出発時刻
        [DataMember] public string ArrvName;  // 到着地名
        [DataMember] public string ArrvTime;  // 到着時刻
        [DataMember] public string LevelCode; // クラスコード("1", "2"など、予約のときに必要)
        [DataMember] public string LevelName; // クラス名(1等、2等など)
        [DataMember] public string Room;      // 空き数 
    }

    //POSTのデータ(ペイロード)にマップするデータ
    [DataContract]
    public class ReservationRuqestInfo
    {
        [DataMember] public string ScheduleCode;
        [DataMember] public string LevelCode;
        [DataMember] public string MemberCode;
        [DataMember] public string LastName;
        [DataMember] public string FirstName;
    }

    //Webページ表示用の最小限の予約データ
    [DataContract]
    public class ReservationBasicInfo   // 1便分のスケジュール
    {
        [DataMember] public string ReservationCode; // 予約番号(予約コード)(削除のときに必要)
        [DataMember] public string MemberCode;      // 会員コード
        [DataMember] public string LastName;        // 姓
        [DataMember] public string FirstName;       // 名
        [DataMember] public string ScheduledDate;   // 運航日
        [DataMember] public string CruiseId;        // 便名
        [DataMember] public string LevelName;       // クラス名
        [DataMember] public string DeptName;  // 出発地名
        [DataMember] public string DeptTime;  // 出発時刻
        [DataMember] public string ArrvName;  // 到着地名
        [DataMember] public string ArrvTime;  // 到着時刻
        [DataMember] public string AppliedPrice;   // 適用料金
        [DataMember] public string AppliedTax;     // 適用消費税
        [DataMember] public string BoardingFlag;   // 乗船受付フラグ("1"は乗船済み)
    }
}
