﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

using Adwk.WebApp.ServiceReference1;
using Adwk.Mileage.Data;

namespace Adwk.WebApp.members
{
    public partial class MileageHistory : System.Web.UI.Page
    {
        // 使用するセッションデータ
        //   Session[SVars.UserName]    ---- ユーザー名(SVars.MemberInfoと同期)
        //   Session[SVars.MemberInfo]  ---- ユーザー情報
        //   Session[SVars.HistoryList] ---- マイル履歴(1か月分)

        // このページで使用するメッセージ
        const string msgNoData = "該当する履歴がありません。";

        // 以下のメンバー変数は 1回のポストバックの中で複数のメソッドから共有可能
        // Loadイベントで値が設定される
        string username = null;           // 使用するユーザー名
        MemberInfo memberInfo = null;     // 使用するユーザー情報

        protected void Page_Load(object sender, EventArgs e)
        {
            bool result;
            MileageQuery2Client proxy = null; // マイル管理サービスのプロキシ

            // 会員情報をセッション変数から取得
            // セッション変数が空の場合で認証済みの場合はセッション変数を構築
            if (Session[SVars.UserName] == null ||
                Session[SVars.MemberInfo] == null)
            {
                // 認証済みであるか念のため確認
                if (!Page.User.Identity.IsAuthenticated)
                {
                    // (現在の構成では、Authorizationの設定によって、
                    //  未認証ではこのページにたどり着くことはないが、
                    // 念のため、未認証ではリダイレクトさせる。
                    Response.Redirect("~/zTimeOut.aspx");
                }
                // セッション変数に退避
                // (マスターページと同じ処理、ただし IMileageQuery2を使う点は異なる)
                proxy = new MileageQuery2Client();

                result = proxy.QueryInfo1ByMember(
                    out memberInfo, Page.User.Identity.Name);

                if (result)
                {
                    //2回目以降は、WCFサービスアクセスを避けるため、ユーザー情報を退避
                    username = memberInfo.LastName + " " + memberInfo.FirstName;
                    Session[SVars.UserName] = username;
                    Session[SVars.MemberInfo] = memberInfo;
                }
                else
                {
                    // Webのログイン ユーザー名(会員コード)に対応する
                    // 会員ユーザー名はあるはずなので、
                    // 本来はここは実行されない。
                    Session[SVars.UserName] = null;
                    Session[SVars.MemberInfo] = null;
                }
            }
            else
            {
                username = Session[SVars.UserName] as string;
                memberInfo = Session[SVars.MemberInfo] as MemberInfo; 
            }

            // 他のページから移動してきたポストバックでない初回表示の初期化
            if (!IsPostBack)
            {
                // ユーザー情報の設定
                UserLabel.Text = username ?? "";
                IDLabel.Text = (memberInfo == null)? "" : memberInfo.MemberCode;
                TotalMileLabel.Text = String.Format("{0:#,##0}", memberInfo.TotalMile);

                // 月を選択する一覧の作成
                MakeupMonthList();
                
                // 選択された月の履歴を作成
                MakeupMonthHistory();

            }

            // 下部メッセージは無条件で初期化
            ShowMessage("");

        }

        // メッセージの表示
        private void ShowMessage(string msg)
        {
            MsgLabel.Text = msg;
        }

        // 月を選択するリストボックスを構築する
        private void MakeupMonthList()
        {
            // ここでは、サンプルデータが 2011/06 が中心なので、
            // この月を含む 12ヶ月(2011/04から)をリストに表示する
            int year = 2011, month = 4; 
            const int MaxMonths = 24;
            const int selectedMonthIndex = 4;
            for (int i = 0; i < MaxMonths; i++, month++)
            {
                if (month > 12) { month = 1; year++; }
                MonthList.Items.Add(
                    new ListItem(
                        String.Format("  {0:0000}年{1:00}月  ", year, month),
                        String.Format("{0:0000}/{1:00}/01", year, month)));
            }
            MonthList.SelectedIndex = selectedMonthIndex;
        }

        // 選択された月に基づいてマイル履歴を構築する
        private void MakeupMonthHistory()
        {
            // 月が選択されない場合や、ユーザー情報が無い場合は何もしない
            if (MonthList.SelectedIndex < 0) return;
            if (memberInfo == null) return;

            // 選択された月から期間を算出
            DateTime startDate, untilDate;
            string selectedDate = MonthList.SelectedValue as String;
            if( selectedDate == null ||
                ! DateTime.TryParse( MonthList.SelectedValue, out startDate))
            {
                return;
            }
            untilDate = startDate.AddMonths(1);

            // マイル管理サービスにアクセスして、マイル履歴を取得
            var proxy = new MileageQuery2Client();
            AdwkMStatus stat;
            string msg;
            AdwkMileagesDataSet.会員マイル履歴DataTable historyTable;
            historyTable = proxy.QueryHistoryByMemberAndPeriod(
                out stat, out msg,
                memberInfo.MemberCode, startDate, untilDate);
            
            // データバインドを行う
            // 履歴がない場合でも、0件のテーブルとバインドすることでクリアできる。
            HistoryGridView.DataSource = historyTable;
            HistoryGridView.DataBind();
            Session[SVars.HistoryList] = historyTable; //特定月の履歴を退避
            
            // メッセージの表示
            string resultMsg = msg;
            if (historyTable.Rows.Count <= 0)
            {
                // データがゼロ件の場合は、その旨のメッセージを追加する
                resultMsg = msgNoData + " " + resultMsg;
            }
            ShowMessage(resultMsg);
        }

        // 月を選択するリストボックスMonthListで選択月が変化
        //   MonthListはAutoPostBackがtruなので、選択を変えたらポストバックする
        //   UpdatePanelの中に配置されているので、AJAXを使用して部分的な更新ができる
        protected void MonthList_SelectedIndexChanged(object sender, EventArgs e)
        {
            MakeupMonthHistory();
        }

        // 履歴一覧のページ切り替え
        protected void HistoryGridView_PageIndexChanging(object sender, GridViewPageEventArgs e)
        {
            // ページ切り替えに伴い、GridViewのステートを更新する必要がある
            // (明示的に行わないと、前のページのコンテンツのまま)

            var historyTable = Session[SVars.HistoryList]
                as AdwkMileagesDataSet.会員マイル履歴DataTable;
            if (historyTable == null)
            {
                // セッション変数のデータが無ければタイムアウトのページへ移動
                Response.Redirect("~/zTimeOut.aspx");
            }
            HistoryGridView.DataSource = historyTable;
            HistoryGridView.PageIndex = e.NewPageIndex;
            HistoryGridView.DataBind();
        }
        protected void HistoryGridView_PageIndexChanged(object sender, EventArgs e)
        {

        }

        // [マイルの確認と使用]メニューへ戻る
        protected void GpBackButton_Click(object sender, EventArgs e)
        {
            Response.Redirect("MileageMain.aspx");
        }


    }
}