﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

using Adwk.WebApp.ServiceReference1;

namespace Adwk.WebApp.members
{
    public partial class MileageTransfer : System.Web.UI.Page
    {
        // 使用するセッションデータ
        //   Session[SVars.UserName]     ---- ユーザー名(SVars.MemberInfoと同期)
        //   Session[SVars.MemberInfo]   ---- ユーザー情報
        //   Session[SVars.TransferMile] ---- 移行マイル数

        // このページで使用するメッセージ
        const string msgInvalidMile    = "移行マイル数が有効な数値ではありません。";
        const string msgZeroOrNegative = "移行マイル数に１マイル未満を指定できません。";
        const string msgMileNotEnough  = "移行マイル数が現在のマイル累計残高を越えています。";

        // 以下のメンバー変数は 1回のポストバックの中で複数のメソッドから共有可能
        // Loadイベントで値が設定される
        string username = null;           // 使用するユーザー名
        MemberInfo memberInfo = null;     // 使用するユーザー情報

        protected void Page_Load(object sender, EventArgs e)
        {
            // 会員情報をセッション変数から取得
            // セッション変数が空の場合はタイムアウトとみなす
            if (Session[SVars.UserName] == null ||
                Session[SVars.MemberInfo] == null)
            {
                Response.Redirect("~/zTimeOut.aspx");
            }
            else
            {
                // セッション変数からユーザー情報を取得
                username = Session[SVars.UserName] as string;
                memberInfo = Session[SVars.MemberInfo] as MemberInfo;
            }
            System.Diagnostics.Debug.Assert(username != null);
            System.Diagnostics.Debug.Assert(memberInfo != null);

            // 他のページから移動してきたポストバックでない初回表示の初期化
            if (!IsPostBack)
            {
                // ユーザー情報の設定
                UserLabel.Text = username;
                UserLabelAlt.Text = username;
                IDLabel.Text = memberInfo.MemberCode;
                TotalMileLabel.Text = String.Format("{0:#,##0}", memberInfo.TotalMile);

                // 移行情報の復元
                // 確認画面から戻る場合は移行情報も復元する
                if (Session[SVars.TransferMile] != null)
                {
                    TransferMileTextBox.Text =
                        Session[SVars.TransferMile].ToString();
                }
                if (Session[SVars.CustomerId] != null)
                {
                    CustomerIdTextBox.Text =
                        Session[SVars.CustomerId].ToString();
                }
            }
        }

        // [確認画面へ進む]ボタン
        protected void ConfirmButton_Click(object sender, EventArgs e)
        {
            // 検証コントロールの(サーバーサイドでの)検証に失敗したら
            // 何もせずに終わる
            // (検証コントロールのエラーメッセージは自動的に表示される)
            if (!Page.IsValid)
            {
                return;
            }

            // サーバーサイドでのカスタム検証
            // 検証コントロールに検証の失敗を設定することで
            // エラーメッセージをValidation Summaryに表示させることができる。
            int transferMile = 0;
            if ( !Int32.TryParse(TransferMileTextBox.Text, out transferMile))
            {
                // 移行マイル数が有効な数値ではない
                CustomValidator1.IsValid = false; //検証失敗
                CustomValidator1.ErrorMessage = msgInvalidMile;
                return;
            }
            if (transferMile <= 0)
            {
                // 移行マイルが 1マイル未満
                CustomValidator1.IsValid = false;
                CustomValidator1.ErrorMessage = msgZeroOrNegative;
                return;
            }
            if (transferMile > memberInfo.TotalMile)
            {
                // 移行マイルの残高不足
                CustomValidator1.IsValid = false;
                CustomValidator1.ErrorMessage = msgMileNotEnough;
                return;
            }

            // 顧客IDは必須項目として検証済み
            System.Diagnostics.Debug.Assert(
                !String.IsNullOrEmpty(CustomerIdTextBox.Text));

            // 移行マイルをセッション変数に退避して確認画面へ
            Session[SVars.TransferMile] = transferMile;
            Session[SVars.CustomerId] = CustomerIdTextBox.Text;
            Response.Redirect("MileageTransferConfirm.aspx");
        }

        // [［マイルの確認と使用］メニューへ戻る]ボタン
        protected void GpBackButton_Click(object sender, EventArgs e)
        {
            // このページは、移行確認画面から戻っている場合があるので、
            // 移行マイルや移行先顧客Idをセッション変数に確保している
            // 可能性がある。メニュー戻るのでクリアする。
            Session[SVars.TransferMile] = null;
            Session[SVars.CustomerId] = null;

            // メニューに戻る
            Response.Redirect("MileageMain.aspx");
        }
    }
}