﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

using Adwk.WebApp.ServiceReference1;
using Adwk.WebApp.ServiceReference2;
using Adwk.Mileage.Data;

namespace Adwk.WebApp.members
{
    public partial class MileageTransferConfirm : System.Web.UI.Page
    {
        // 使用するセッションデータ
        //   Session[SVars.UserName]     ---- ユーザー名(SVars.MemberInfoと同期)
        //   Session[SVars.MemberInfo]   ---- ユーザー情報
        //   Session[SVars.TransferMile] ---- 移行マイル数
        //   Session[SVars.CustomerId]   ---- 移行先の顧客ID(お客様ID)

        // このページで使用するメッセージ
        const string msgInternalError = "社内システムでエラーが発生しました。";
        const string msgAdditionalOnError = "キャンセルして、しばらく経ってから移行手続きを行なってみてください。";

        // 以下のメンバー変数は 1回のポストバックの中で複数のメソッドから共有可能
        // Loadイベントで値が設定される
        string username = null;       // 使用するユーザー名
        MemberInfo memberInfo = null; // 使用するユーザー情報
        int transferMile = 0;         // 移行マイル数
        string customerId = null;     // 移行先の顧客ID

        protected void Page_Load(object sender, EventArgs e)
        {
            // 会員情報をセッション変数から取得
            // セッション変数が空の場合はタイムアウトとみなす
            if (Session[SVars.UserName] == null ||
                Session[SVars.MemberInfo] == null ||
                Session[SVars.TransferMile] == null ||
                Session[SVars.CustomerId] == null)
            {
                Response.Redirect("~/zTimeOut.aspx");
            }
            else
            {
                // セッション変数からユーザー情報を取得
                username = Session[SVars.UserName] as string;
                memberInfo = Session[SVars.MemberInfo] as MemberInfo;
                try
                {
                    transferMile = (int)Session[SVars.TransferMile];
                }
                catch
                { }
                customerId = Session[SVars.CustomerId] as string;
            }
            System.Diagnostics.Debug.Assert(username != null);
            System.Diagnostics.Debug.Assert(memberInfo != null);
            System.Diagnostics.Debug.Assert(transferMile > 0);
            System.Diagnostics.Debug.Assert(customerId != null);

            // 他のページから移動してきたポストバックでない初回表示の初期化
            if (!IsPostBack)
            {
                // ユーザー情報の設定
                UserLabel.Text = username;
                UserLabelAlt.Text = username;
                IDLabel.Text = memberInfo.MemberCode;
                TotalMileLabel.Text = String.Format("{0:#,##0}", memberInfo.TotalMile);

                // 移行情報の設定
                TransferMileLabel.Text =
                    String.Format("{0:#,##0}", transferMile);
                CustomerIdLabel.Text = customerId;
            }
        }

        // [確定]ボタン
        protected void RunButton_Click(object sender, EventArgs e)
        {
            // ポストバック時、Loadイベントハンドラーの中で設定されてるいはず
            System.Diagnostics.Debug.Assert(username != null);
            System.Diagnostics.Debug.Assert(memberInfo != null);
            System.Diagnostics.Debug.Assert(transferMile > 0);
            System.Diagnostics.Debug.Assert(customerId != null);

            // マイレージ管理サービスの移行処理の呼び出し
            var proxy1 = new MileageTransferClient();
            bool result;
            AdwkMStatus stat;
            string msg;

            result = proxy1.TransferMileByMember(
                        out stat, out msg,
                        memberInfo.MemberCode,
                        customerId,
                        DateTime.Now.Date,  //マイル移行日はシステム日付(サンプル実行日)
                        transferMile);
            proxy1.Close();

            // メッセージの表示とUIの更新
            switch (stat)
            {
                // 移行処理成功
                case AdwkMStatus.Success:
                    MessageLabel1.Text = msg;
                    MessageLabel2.Text = "";
                    StepBackButton.Visible = false; // [戻る]ボタン非表示
                    RunButton.Visible = false;      // [確定]ボタン非表示
                    break;
                // 外部システムでのエラー
                case AdwkMStatus.ExternalError:
                    MessageLabel1.Text = msg;
                    MessageLabel2.Text = "";
                    break;
                // 内部システムでのエラー
                default:
                    MessageLabel1.Text = msgInternalError;
                    MessageLabel2.Text = msgAdditionalOnError;
                    break;
            }

            // マイル累計更新によるユーザー情報の更新
            if (stat == AdwkMStatus.Success)
            {
                // ユーザー情報をマイレージ管理サービスから取得
                var proxy2 = new MileageQueryClient();
                result = proxy2.QueryInfo1ByMember(
                    out memberInfo, Page.User.Identity.Name);
                proxy2.Close();

                // ユーザー情報のセッション変数を更新
                if (result)
                {
                    // セッション変数に退避することで、他のページに移動すると、
                    // その新しいマイル数を表示できる
                    username = memberInfo.LastName + " " + memberInfo.FirstName;
                    Session[SVars.UserName] = username;
                    Session[SVars.MemberInfo] = memberInfo;
                    // 同じユーザーの情報を取得しているはず
                    System.Diagnostics.Debug.Assert(
                        UserLabel.Text == username);
                }
                else
                {
                    Session[SVars.UserName] = username;
                    Session[SVars.MemberInfo] = memberInfo;
                }

            }    
        }
        
        // [戻る]ボタン
        protected void StepBackButton_Click(object sender, EventArgs e)
        {
            // 移行マイルと移行顧客先Idはセッション変数に維持されているので
            // マイル指定画面では、これらの値を復元表示できる

            // マイル指定画面に戻る
            Response.Redirect("MileageTransfer.aspx");
        }

        // [［マイルの確認と使用］メニューへ戻る]ボタン
        protected void GpBackButton_Click(object sender, EventArgs e)
        {
            // 移行手続きのキャンセルなので、
            // 移行マイルや移行先顧客Idをセッション変数クリアする。
            Session[SVars.TransferMile] = null;
            Session[SVars.CustomerId] = null;

            // メニューに戻る
            Response.Redirect("MileageMain.aspx");
        }

    }
}