﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.Text;
using System.ServiceModel.Activation;

using Adwk.Reservation.Logic;
using Adwk.Reservation.Data;

using System.Web;  // HttpContext
using Adwk.WebApp.ServiceReference1;  // MemberInfo
using System.ServiceModel.Web;        // WebFaultException
using Adwk.Reservation;  // ReservationInfo    

namespace Adwk.WebApp
{
    [AspNetCompatibilityRequirements(
        RequirementsMode = AspNetCompatibilityRequirementsMode.Allowed)]
    public class ReservationService : IReservation
    {
        // 運行一覧の検索
        ScheduleBasicInfo[] IReservation.QuerySchedule(
            DateTime thisDate, string deptCode, string arrvCode, string levelCode)
        {
            // HTTP GET の際に、ASP.NETキャッシュを使用させないため
            HttpContext.Current.Response.Cache.SetCacheability(
                HttpCacheability.NoCache);

            // 翌日の0時0分を算出
            var untilDate = new DateTime(
                thisDate.Year, thisDate.Month, thisDate.Day);
            untilDate += new TimeSpan(1, 0,0,0);
            
            // 検索(簡単にするため、該当無し、例外に関わらず
            //      ゼロ個の配列を返す)
            ScheduleBasicInfo[] sbinfos = null;

            // 運行データの取得
            Reservation.ScheduleInfo[] sinfos = null;
            Reservation.Data.AdwkRStatus stat;
            string msg = null;
            try
            {
                bool result = BLComp2.QuerySchedule(
                    thisDate, untilDate, deptCode, arrvCode, levelCode,
                    out sinfos, out stat, out msg);
            }
            catch
            {
            }

            // 返すべきデータの構築
            if (sinfos != null)
            {
                var query = from sinfo in sinfos
                            select new ScheduleBasicInfo()
                            {
                                ScheduleCode = sinfo.ScheduleCode,
                                ScheduledDate = String.Format("{0:yyyy-MM-dd}",
                                                    sinfo.ScheduledDate),
                                CruiseId = sinfo.CruiseId,
                                DeptName = sinfo.DeptName,
                                DeptTime = String.Format("{0:00}:{1:00}",
                                    sinfo.DeptTime.Hours, sinfo.DeptTime.Minutes),
                                ArrvName = sinfo.ArrvName,
                                ArrvTime = String.Format("{0:00}:{1:00}", 
                                    sinfo.ArrvTime.Hours, sinfo.ArrvTime.Minutes),
                                LevelCode = sinfo.LevelCode,
                                LevelName = sinfo.LevelName,
                                Room = sinfo.Room.ToString()
                            };
                sbinfos = query.ToArray();
            }
            else
                sbinfos = new ScheduleBasicInfo[0];

            return sbinfos;
        }

        // Webからの予約登録
        string IReservation.ReserveByWeb(ReservationRuqestInfo reqInfo)
        {
            const string formatSuccess = "登録が完了しました。予約番号は{0}です。";
            const string formatNoRoom  = "恐れ入りますが、空席が無くなりました。" +
                                         "初めからやり直してください。。";
            const string formatErr = "恐れ入りますが、予約手続きができません。" +
                                     "初めからやり直してください。";

            string reservationCode;
            AdwkRStatus stat;
            string msg;
            bool result;
            result = BLComp2.Reserve(reqInfo.ScheduleCode, reqInfo.LevelCode,
                        reqInfo.MemberCode, reqInfo.LastName, reqInfo.FirstName,
                        out reservationCode, out stat, out msg);

            if (result)
            {
                if (stat == AdwkRStatus.NoRoom)
                {
                    return formatNoRoom;
                }
                return String.Format(formatSuccess, reservationCode);
            }
            else
            {
                return formatErr;
            }
        }

        // Webからの予約確認
        //   (Webにログインしているユーザーに関して予約データを検索)
        //   (予約が見つからない場合はnullを返す)
        ReservationBasicInfo IReservation.QueryReservation1ByRCode(string reservationCode)
        {
            // HTTP GET の際に、ASP.NETキャッシュを使用させないため
            HttpContext.Current.Response.Cache.SetCacheability(
                HttpCacheability.NoCache);

            try
            {
                // ASP.NETレベルのセッションが保持するユーザー情報を取得
                HttpContext context1 = HttpContext.Current;
                MemberInfo minfo = (MemberInfo)context1.Session[SVars.MemberInfo];

                // コンポーネントがなければ作成(セッション単位で維持)
                BLComp3 comp3 = context1.Session[SVars.BLComp3] as BLComp3;
                if (comp3 == null)
                {
                    comp3 = new BLComp3();
                    context1.Session[SVars.BLComp3] = comp3;
                }

                // 予約データの取得
                ReservationInfo rinfo;
                AdwkRStatus stat;
                string msg;
                bool result;
                result = comp3.QueryReservation1ByRCode(
                            reservationCode, minfo.LastName, minfo.FirstName,
                            out rinfo, out stat, out msg);
                if (!result || stat != AdwkRStatus.Success)
                {
                    return null;
                }
                else
                {
                    // 正常時の処理
                    var rbinfo = new ReservationBasicInfo()
                        {
                            ReservationCode = rinfo.ReservationCode,
                            MemberCode = rinfo.MemberCode,
                            LastName = rinfo.LastName,
                            FirstName = rinfo.FirstName,
                            ScheduledDate = String.Format(
                                "{0:yyyy-MM-dd}",rinfo.ScheduleDate),
                            CruiseId = rinfo.CruiseId,
                            LevelName = rinfo.LevelName,
                            DeptName = rinfo.DeptName,
                            DeptTime = String.Format("{0:00}:{1:00}",
                                rinfo.DeptTime.Hours, rinfo.DeptTime.Minutes),
                            ArrvName = rinfo.ArrvName,
                            ArrvTime = String.Format("{0:00}:{1:00}",
                                rinfo.ArrvTime.Hours, rinfo.ArrvTime.Minutes),
                            AppliedPrice = String.Format(
                                "{0:#,##0}", rinfo.AppliedPrice),
                            AppliedTax = String.Format(
                                "{0:#,##0}", rinfo.AppliedTax),
                            BoardingFlag = rinfo.BoardingFlag
                        };

                    // ==== WCF RESTクライアント向けオプティミスティック同時実行制御の例 ====
                    Guid newETag = Guid.NewGuid();
                    context1.Session[SVars.CurrentETag] = newETag;
                    WebOperationContext.Current.OutgoingResponse.SetETag(newETag);               
                    // ====

                    return rbinfo;
                }
            }
            catch
            {
                // セッションタイムアウトほか、すべて InernalServeErrorとして返す
                throw new WebFaultException(
                    System.Net.HttpStatusCode.InternalServerError);
            }
        }

        // Webからの削除
        // (セッションを維持し、QueryReservation1ByRCodeで予約照会したデータを
        //  維持していることが前提)
        string IReservation.DeleteReservationByRCode(string reservationCode)
        {
            const string formatSuccess = "予約番号 {0} のデータを削除しました。";

            // ASP.NETレベルのセッションが保持するユーザー情報を取得
            HttpContext context1 = HttpContext.Current;

            try
            {
                // コンポーネントを取得(セッション単位で維持)
                BLComp3 comp3 = context1.Session[SVars.BLComp3] as BLComp3;
                if (comp3 == null)
                {
                    // セッション変数の中に、参照で使用したBLComp3が
                    // 見つからない場合は、タイムアウトとして例外にする
                    throw new WebFaultException(
                        System.Net.HttpStatusCode.InternalServerError);
                }

                // ==== WCF RESTクライアント向けオプティミスティック同時実行制御の例 ====
                // 直近で参照したデータのETag(サーバー側で控えたもの)と、
                // クライアントが If-Matchヘッダーで送信したものが同一であるか確認
                Guid currentETag;
                try
                {
                    // この例では実装していないが、
                    // コンポーネント側でデータを更新した場合、
                    // 更新後のデータに紐付けた新しいGUIDになっている
                    currentETag = (Guid)context1.Session[SVars.CurrentETag];
                }
                catch
                {
                    // タイムアウトなどが原因で、控えていたETagが見つからない場合
                    throw new WebFaultException(
                        System.Net.HttpStatusCode.InternalServerError);
                }
                // 次のメソッドで、クライアントからの If-MatchヘッダーのETag値をチェック
                // 一致しない場合は、
                //    WebFaultException(System.Net.HttpStatusCode.PreconditionFailed)
                // をスローする。このため、これ以降は実行されない。
                // ★検証する場合は、次の2行のコメントを解除してください★
                //WebOperationContext opContext1 = WebOperationContext.Current;
                //opContext1.IncomingRequest.CheckConditionalUpdate(currentETag);
                // ====

                // 予約データの削除
                AdwkRStatus stat;
                string msg;
                bool result;
                result = comp3.DeleteReservationByRCode(
                            reservationCode, out stat, out msg);
                if (result)
                {
                    return String.Format(formatSuccess, reservationCode);
                }
                else
                {
                    switch (stat)
                    {
                        // ==== WCF RESTクライアント向けオプティミスティック同時実行制御の例
                        // (データ競合発生時の、試験的なクライアントへの通知)
                        // 予約データを確認中に、他のクライアントから乗船済みにされた場合
                        // BLComp3はデータの競合を検出
                        // (本来、この例では中間層のBLComp3でデータ競合について対処済みであるので
                        //  単にエラーメッセージを伝えれば十分であり、
                        //  クライアントへ、ステータスコード412(Precondition Failed)を伝える必要はありませんが、
                        //  実験のため、あえてこのスタータスコードを返しています。)
                        case AdwkRStatus.OptimisticConcurrencyError:
                            throw new WebFaultException(
                                System.Net.HttpStatusCode.PreconditionFailed);
                        // ====

                        // その他の削除エラーは InternalServerErrorとして返す
                        default:
                            throw new WebFaultException(
                                System.Net.HttpStatusCode.InternalServerError);
                    }
                }
            }
            catch (WebFaultException)
            {
                // すでに WebFaultException 例外が発生していれば、
                // ステータスコードが設定済みなので、そのまま例外をスローする
                throw;
            }
            catch
            {
                // 例外発生は、すべて InternalServeErrorとして返す
                throw new WebFaultException(
                    System.Net.HttpStatusCode.InternalServerError);
            }
        }

        // (テスト用メソッド)サービスインスタンスのハッシュ値を調べる
        // (ReservationConfirm.aspx にコメント化された、呼び出し用のボタンあり)
        int IReservation.GetInstanceHashCode()
        {
            // HTTP GET の際に、ASP.NETキャッシュを使用させないため
            HttpContext.Current.Response.Cache.SetCacheability(
                HttpCacheability.NoCache);

            return this.GetHashCode();
        }
    }
}
