﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.Data.SqlClient; // SQL Server 向けデータプロバイダー
using System.Data;           // SQL Server データ型
using System.Transactions;   // TransactionScopeクラス

using AdwkDac.Properties;    // Settingsクラス

namespace AdwkDac
{
    // マイレージデータ(ポイントデータ)操作のための
    // データアクセスコンポーネント
    public static class DAHelper0
    {
        //ユーザー作成(会員作成)
        public static bool CreateMileUser(
            string memberCode, string lastName, string firstName, out string errorMessage)
        {
            bool result = false;

            var conn = new SqlConnection(Settings.Default.AdwkMileagesConnectionString);
            var insert1 = new SqlCommand();  // [会員]テーブル新規挿入
            var insert2 = new SqlCommand();  // [会員マイレージ]テーブル新規挿入用

            //パラメーターの初期化
            insert1.CommandText =
                "INSERT INTO [会員]([会員コード], [姓], [名])" +
                " VALUES(@membercode, @lname, @fname)";
            insert1.Parameters.Add("@membercode", SqlDbType.NVarChar);
            insert1.Parameters.Add("@lname", SqlDbType.NVarChar);
            insert1.Parameters.Add("@fname", SqlDbType.NVarChar);

            insert2.CommandText =
                "INSERT INTO [会員マイレージ]([会員コード], [マイル累計], [最終更新日時])" +
                " VALUES(@membercode, @total, @updated)";
            insert2.Parameters.Add("@membercode", SqlDbType.NVarChar);
            insert2.Parameters.Add("@total", SqlDbType.Int);
            insert2.Parameters.Add("@updated", SqlDbType.DateTime);

            //会員作成
            try
            {
                using (var scope = new TransactionScope(TransactionScopeOption.Required))
                {
                    conn.Open();
                    insert1.Connection = conn;
                    insert2.Connection = conn;

                    insert1.Parameters["@membercode"].Value = memberCode;
                    insert1.Parameters["@lname"].Value = lastName;
                    insert1.Parameters["@fname"].Value = firstName;
                    insert1.ExecuteNonQuery();

                    insert2.Parameters["@membercode"].Value = memberCode;
                    insert2.Parameters["@total"].Value = 0;
                    insert2.Parameters["@updated"].Value = DateTime.Now;
                    insert2.ExecuteNonQuery();

                    scope.Complete();
                }
                errorMessage = null;
                result = true;
            }
            catch(Exception ex)
            {
                errorMessage = ex.Message + 
                    "(" + ex.GetType().Name + "), DAHelper1.CreateMileUser()";
                result = false;
            }
            finally
            {
                conn.Close();
            }

            return result;
        }

    }
}
