﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

using System.Transactions; // TransactionScope
using System.Web.Security; // Membership
using AdwkDac;             // DAHelper1

namespace AdwkWebApp.Staff
{
    //リダイレクトなどで利用する各ページのURL
    public static class StaffPages
    {
        public const string CreateUserUrl = "CreateUser.aspx";
        public const string CreateUserDoneUrl = "CreateUserDone.aspx";
    }

    //Webアプリ固有のビジネスロジック
    public static class BLComp1
    {
        public static bool CreateUser(
            string userName, string password, string email,
            string passwordQuestion, string passwordAnswer,
            string lastName, string firstName, 
            out string message)
        {
            const string messageOK = "ユーザー作成に成功しました。";
            const string messageNG = "ユーザー作成に失敗しました。内部ステータス:{0}";
            MembershipCreateStatus status;
            bool result = false;

            using(TransactionScope scope = 
                new TransactionScope(TransactionScopeOption.Required))
            {
                // ASP.NETのメンバーシッププロバイダーのユーザー作成
                MembershipUser user;
                user = Membership.CreateUser(
                    userName, password, email,
                    passwordQuestion, passwordAnswer, true,
                    out status);
                if (status == MembershipCreateStatus.Success)
                {
                    // ASP.NETのメンバーシッププロバイダーのユーザー名(userName)は、
                    // マイルデータの会員コード(userCode)として使用しています。
                    string errorMessage;
                    if (DAHelper0.CreateMileUser(
                        userName, lastName, firstName, out errorMessage))
                    {
                        //作成正常終了
                        scope.Complete();
                        result = true;
                        message = messageOK;
                    }
                    else
                    {
                        //作成失敗
                        result = false;
                        //(注) 変数 errorMessage はシステム内部のエラーメッセージですが、
                        //    ここではサンプルプログラムの動作を検証しやすくするため、
                        //    そのままエラーメッセージをフォームに表示します。
                        message = errorMessage;
                    }
                }
                else
                {
                    //(注) 変数 status はシステム内部のエラーメッセージを表すが、
                    //    ここではサンプルプログラムの動作を検証しやすくするため、
                    //    そのままエラーメッセージをフォームに表示します。
                    message = String.Format(messageNG, status.ToString());
                }
            }
            
            return result;
        }
    }

}