﻿// Note.
//   Silverlight 3 アプリケーションの既定のプロジェクトテンプレートを使用し、
// 以下のアセンブリの参照をさらに追加しています。
//     System.Runtime.Serialization.dll
//     System.ServiceModel.web.dll
//     System.Windows.Controls.dll
//     System.Windows.Controls.Data.dll

using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;

using Adwk.SLApp.Reservation;
using Adwk.SLApp.WebApp;
using Adwk.WebApp;

using System.IO;
using System.Runtime.Serialization.Json;
using System.Text;
using System.Runtime.Serialization;

namespace SilverAppReservation
{
    public partial class MainPage : UserControl
    {
        // メッセージの文字列
        const string msgNone = "";
        const string msgSelectPromttUpper = "ご希望の日付の経路とクラスを選択してください。";
        const string msgSelectPromptLower = "予約する運行便を上記の画面からご指定ください。 ";
        const string msgSearchInProgress = "ご指定の運行便を検索中です...";
        const string msgNotAvailable = "該当する予約可能な運行便がございません。";
        const string msgCheckUserInfo = "お客様の情報を確認しています...";

        public MainPage()
        {
            InitializeComponent();

            // コントロールの初期データ設定
            dateSelect.SelectedDate = new DateTime(2011, 7, 1);
            LoadRBasicInfo();   
        }

        // 基本情報(路線とクラス)の読み込み
        private void LoadRBasicInfo()
        {
            var srvUri = new Uri(String.Format(
                "http://{0}:{1}/AdwkWebApp/RBasicInfoService.svc/GetBasicRouteInfo",
                Application.Current.Host.Source.DnsSafeHost,
                Application.Current.Host.Source.Port));
            var client1 = new WebClient();
            client1.OpenReadCompleted +=
                new OpenReadCompletedEventHandler(client1_OpenReadCompleted);
            client1.OpenReadAsync(srvUri);
        }

        // 基本情報の読み込み(路線とクラス)完了時のイベントハンドラー
        private void client1_OpenReadCompleted(
            object sender, OpenReadCompletedEventArgs e)
        {
            if (e.Error != null)
            {
                MessageBox.Show(e.Error.Message + "\n" + e.Error.GetType().Name);
                return;
            }

            // 応答のStream参照
            Stream stream1 = e.Result;

            // JSON文字列の逆シリアル化
            var serializer = new DataContractJsonSerializer(typeof(BasicInfos));
            BasicInfos basicInfos = (BasicInfos)serializer.ReadObject(stream1);

            // 戻り値と出力引数の取り出し
            bool retVal = basicInfos.GetBasicRouteInfoResult;
            PlaceInfo[] places = basicInfos.places;
            LevelInfo[] levels = basicInfos.levels;

            // 戻り値の確認
            if (!retVal)
            {
                MessageBox.Show(
                    "恐れ入りますが、現在、予約サービスをご利用できません。");
                return;
            }

            // 各リストボックスにデータを設定
            deptList.DisplayMemberPath = "PlaceName";
            deptList.ItemsSource = places;

            arrvList.DisplayMemberPath = "PlaceName";
            arrvList.ItemsSource = places;

            classList.DisplayMemberPath = "LevelName";
            classList.ItemsSource = levels;

            // 読み込み完了時に準備完了の初期メッセージ表示
            message1.Text = msgSelectPromttUpper;
            message3.Text = msgSelectPromptLower;
            message3.Visibility = Visibility.Visible;

            // DataGridのクリア
            scheduleDataGrid.ItemsSource = null;
            scheduleDataGrid.Visibility = Visibility.Collapsed;

            // ページ切り替え
            subTitle1.Text = "運行予定一覧";
            page0Init.Visibility = Visibility.Collapsed;
            page1List.Visibility = Visibility.Visible;
            page2Confirm.Visibility = Visibility.Collapsed;
            page2Registering.Visibility = Visibility.Collapsed;
            page3Done.Visibility = Visibility.Collapsed;

            // テスト用の応答文字列表示コード
            //if (stream1.CanSeek)
            //{
            //    stream1.Seek(0, SeekOrigin.Begin);
            //    var sr = new StreamReader(stream1, Encoding.UTF8);
            //    MessageBox.Show(sr.ReadToEnd());
            //}
        }

        // 検索条件の選択に伴い、スケジュールの検索
        private void OnSelectChanged(object sender, SelectionChangedEventArgs e)
        {
            // すべての項目で選択されていなければ、検索を行わない
            if (dateSelect.SelectedDate == null) return;
            if (deptList.SelectedItem == null) return;
            if (arrvList.SelectedItem == null) return;
            if (classList.SelectedItem == null) return;

            // 選択項目の取り出し
            string dateVal = String.Format(                // 日付
                "{0:yyyy-MM-dd}", dateSelect.SelectedDate);
            PlaceInfo deptInfo = deptList.SelectedItem as PlaceInfo;
            if (deptInfo == null) return;
            string deptVal = deptInfo.PlaceCode;   // 出発地
            PlaceInfo arrvInfo = arrvList.SelectedItem as PlaceInfo;
            if (arrvInfo == null) return;
            string arrvVal = arrvInfo.PlaceCode;   // 到着地
            LevelInfo classInfo = classList.SelectedItem as LevelInfo;
            if (classInfo == null) return;
            string classVal = classInfo.LevelCode; // クラス

            // スケジュールの検索開始
            var srvUri = new Uri(String.Format(
                 "http://{0}:{1}/AdwkWebApp/members/ReservationService.svc" +
                 "/QuerySchedule?date={2}&dept={3}&arrv={4}&level={5}",
                 Application.Current.Host.Source.DnsSafeHost,
                 Application.Current.Host.Source.Port,
                 dateVal, deptVal, arrvVal, classVal));
            var client2 = new WebClient();
            client2.OpenReadCompleted +=
                new OpenReadCompletedEventHandler(client2_OpenReadCompleted);
            client2.OpenReadAsync(srvUri);

            // 検索開始時のメッセージ表示
            message3.Text = msgSearchInProgress;
            message3.Visibility = Visibility.Visible;
            // DataGridのクリア
            scheduleDataGrid.ItemsSource = null;
            scheduleDataGrid.Visibility = Visibility.Collapsed;

            // ページ切り替え
            subTitle1.Text = "検索中...";
            page0Init.Visibility = Visibility.Collapsed;
            page1List.Visibility = Visibility.Visible;
            page2Confirm.Visibility = Visibility.Collapsed;
            page2Registering.Visibility = Visibility.Collapsed;
            page3Done.Visibility = Visibility.Collapsed;
        }

         // スケジュール検索の完了時のイベントハンドラー
        void client2_OpenReadCompleted(
            object sender, OpenReadCompletedEventArgs e)
        {
            if (e.Error != null)
            {
                MessageBox.Show(e.Error.Message + "\n" + e.Error.GetType().Name);

                // 再び、運行便の指定検索から行う
                OnSelectChanged(sender, null);

                return;
            }

            // 応答のStream参照
            Stream stream1 = e.Result;
            // 運行予定一覧のUI構築
            BuildScheduleList(stream1);
        }

        // 運行予定一覧のUI構築
        private void BuildScheduleList(Stream stream1)
        {
            // JSON文字列の逆シリアル化
            var serializer = new DataContractJsonSerializer(typeof(ScheduleBasicInfo[]));
            ScheduleBasicInfo[] schedules =
                (ScheduleBasicInfo[])serializer.ReadObject(stream1);

            // UI構築
            if (schedules != null && schedules.Length > 0)
            {
                message3.Text = msgNone;
                message3.Visibility = Visibility.Collapsed;
                scheduleDataGrid.ItemsSource = schedules;
                scheduleDataGrid.Visibility = Visibility.Visible;
            }
            else
            {
                message3.Text = msgNotAvailable;
                message3.Visibility = Visibility.Visible;
                scheduleDataGrid.ItemsSource = null;
                scheduleDataGrid.Visibility = Visibility.Collapsed;
            }
           
            // ページ切り替え
            subTitle1.Text = "運行予定一覧";
            page0Init.Visibility = Visibility.Collapsed;
            page1List.Visibility = Visibility.Visible;
            page2Confirm.Visibility = Visibility.Collapsed;
            page2Registering.Visibility = Visibility.Collapsed;
            page3Done.Visibility = Visibility.Collapsed;

            // テスト用の応答文字列表示コード
            //if (stream1.CanSeek)
            //{
            //    stream1.Seek(0, SeekOrigin.Begin);
            //    var sr = new StreamReader(stream1, Encoding.UTF8);
            //    MessageBox.Show(sr.ReadToEnd());
            //}
        }

        // 予約確認
        private void ConfirmRegister(object sender, RoutedEventArgs e)
        {
            Button button1 = sender as Button;
            if (button1 == null) return;

            // ButtonのTagプロパティには、行ごとに ScheduleBasicinfoオブジェクトが
            // バインディングされている
            targetSchedule = button1.Tag as ScheduleBasicInfo;

            // 現在のユーザー情報取得の開始
            var srvUri = new Uri(String.Format(
                "http://{0}:{1}/AdwkWebApp/WebBasicInfoService.svc/GetWebUserInfo",
                 Application.Current.Host.Source.DnsSafeHost,
                 Application.Current.Host.Source.Port));
            var client3 = new WebClient();
            client3.OpenReadCompleted +=
                new OpenReadCompletedEventHandler(client3_OpenReadCompleted);
            client3.OpenReadAsync(srvUri);

            // ユーザー開始時のメッセージ表示
            message3.Text = msgCheckUserInfo;
            message3.Visibility = Visibility.Visible;
            // DataGridの非表示
            scheduleDataGrid.Visibility = Visibility.Collapsed;

            // ページ切り替え
            subTitle1.Text = "確認中...";
            page0Init.Visibility = Visibility.Collapsed;
            page1List.Visibility = Visibility.Visible;
            page2Confirm.Visibility = Visibility.Collapsed;
            page2Registering.Visibility = Visibility.Collapsed;
            page3Done.Visibility = Visibility.Collapsed;
        }

        // 現在のユーザー情報取得の完了時のイベントハンドラー
        // (および予約確認の表示)
        void client3_OpenReadCompleted(
            object sender, OpenReadCompletedEventArgs e)
        {
            if (e.Error != null)
            {
                MessageBox.Show(e.Error.Message + "\n" + e.Error.GetType().Name);

                // 再び、運行便の指定検索から行う
                OnSelectChanged(sender, null);
                
                return;
            }

            // 応答ストリームの参照
            Stream stream1 = e.Result;

            // 現在のユーザー情報の取り出し
            // JSON文字列の逆シリアル化
            var serializer = new DataContractJsonSerializer(typeof(WebUserInfo));
            WebUserInfo uinfo = (WebUserInfo) serializer.ReadObject(stream1);

            // 予約確認情報の退避
            scheduleCode = targetSchedule.ScheduleCode;
            levelCode = targetSchedule.LevelCode;
            memberCode = uinfo.MemberCode;
            lastName = uinfo.LastName;
            firstName = uinfo.FirstName;

            // UI構築
            confirmName.Text = lastName + " " + firstName;
            confirmCode.Text = memberCode;
            confirmDate.Text = targetSchedule.ScheduledDate;
            confirmRoute.Text = targetSchedule.DeptName + " ＞＞ " +
                                targetSchedule.ArrvName;
            confirmTime.Text = targetSchedule.DeptTime;
            confirmClass.Text = targetSchedule.LevelName;

            // ページ表示切替
            subTitle1.Text = "予約確認";
            page0Init.Visibility = Visibility.Collapsed;
            page1List.Visibility = Visibility.Collapsed;
            page2Confirm.Visibility = Visibility.Visible;
            page3Done.Visibility = Visibility.Collapsed;
        }

        // 予約確認情報退避
        private ScheduleBasicInfo targetSchedule;
        private string scheduleCode;
        private string levelCode;
        private string memberCode;
        private string lastName;
        private string firstName;

        // 予約確認時の[戻る]ボタン
        private void backButton_Click(object sender, RoutedEventArgs e)
        {
            // 再び、運行便の指定検索から行う
            OnSelectChanged(sender, null);
        }

        // 予約確認時の[予約]ボタン
        private void reserveButton_Click(object sender, RoutedEventArgs e)
        {
            MessageBox.Show(
                "恐れ入りますが、このページからの予約登録は、" +
                "現在行っておりません。");
        }

    }

    // 応答のJSON文字列を逆シリアル化した際に使用するクラス
    [DataContract]
    public class BasicInfos
    {
        [DataMember] public bool GetBasicRouteInfoResult;
        [DataMember] public PlaceInfo[] places;
        [DataMember] public LevelInfo[] levels;
    }
}
