﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Linq;
using System.Windows;

using System.Windows.Data; //コンバーターで使用
using System.Globalization;

namespace Adwk.WpfApp
{
    /// <summary>
    /// App.xaml の相互作用ロジック
    /// </summary>
    public partial class App : Application
    {
    }

    // 日付のコンバーター(yyyy/MM/dd形式の文字列に変換)
    [ValueConversion(typeof(DateTime), typeof(String))]
    public class DateConverter : IValueConverter
    {
        public object Convert(
            object value, Type targetType, 
            object parameter, CultureInfo culture)
        {
            DateTime date = (DateTime)value;
            return String.Format("{0:yyyy/MM/dd}", date);
        }

        public object ConvertBack(
            object value, Type targetType,
            object parameter, CultureInfo culture)
        {
            string strValue = value as string;
            DateTime resultDateTime;
            if (DateTime.TryParse(strValue, out resultDateTime))
            {
                return resultDateTime;
            }
            return DependencyProperty.UnsetValue;
        }
    }

    // 時間のコンバーター(hh:mm形式の文字列に変換)
    [ValueConversion(typeof(TimeSpan), typeof(String))]
    public class TimeConverter : IValueConverter
    {
        public object Convert(
            object value, Type targetType,
            object parameter, CultureInfo culture)
        {
            TimeSpan time = (TimeSpan)value;
            return String.Format("{0:00}:{1:00}", time.Hours, time.Minutes);
        }

        public object ConvertBack(
            object value, Type targetType,
            object parameter, CultureInfo culture)
        {
            string strValue = value as string;
            TimeSpan resultDateTime;
            if (TimeSpan.TryParse(strValue, out resultDateTime))
            {
                return resultDateTime;
            }
            return DependencyProperty.UnsetValue;
        }
    }

    // 数値のコンバーター(#,##0形式の文字列に変換)
    [ValueConversion(typeof(decimal), typeof(String))]
    public class NumericConverter : IValueConverter
    {
        public object Convert(
            object value, Type targetType,
            object parameter, CultureInfo culture)
        {
            decimal amount = (decimal)value;
            return String.Format("{0:#,##0}", amount);
        }

        public object ConvertBack(
            object value, Type targetType,
            object parameter, CultureInfo culture)
        {
            string strValue = value as string;
            decimal amount;
            if (decimal.TryParse(strValue, out amount))
            {
                return amount;
            }
            return DependencyProperty.UnsetValue;
        }
    }

    // 状況表示のコンバーター
    [ValueConversion(typeof(String), typeof(String))]
    public class StatusConverter : IValueConverter
    {
        public object Convert(
            object value, Type targetType,
            object parameter, CultureInfo culture)
        {
            string stat = value as String;
            switch (stat)
            {
                case "0": return "予約";
                case "1": return "乗船済み";
                default: return "-";
            }
        }

        public object ConvertBack(
            object value, Type targetType,
            object parameter, CultureInfo culture)
        {
            string stat = value as String;
            switch (stat)
            {
                case "予約": return "0";
                case "乗船済み": return "1";
                default: break;
            }
            return DependencyProperty.UnsetValue;
        }
    }

}
