﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

using Adwk.WpfApp.ServiceReference1;
using Adwk.WpfApp.ServiceReference2;

namespace Adwk.WpfApp
{
    /// <summary>
    /// W01Reserve.xaml の相互作用ロジック
    /// </summary>
    public partial class W01Reserve : Window
    {
        public W01Reserve()
        {
            InitializeComponent();

            // ユーザーインターフェイスに必要な予約関連情報の取得と設定
            BuildUI();
        }

        // ユーザーインターフェイスに必要な予約関連情報の取得と設定
        private void BuildUI()
        {
            // 経路とクラス情報を取得してコンボボックスに設定する
            var proxy = new RBasicInfoClient();

            // 明示的な Windows ユーザーアカウントの指定
            // (このプロキシがアクセスする基本情報のサービスでは、
            // 認証を使用していないので、このクレデンシャルは設定しても利用されない)
            //proxy.ClientCredentials.Windows.ClientCredential.UserName = "Administrator";
            //proxy.ClientCredentials.Windows.ClientCredential.Password = "p@ssw0rd";
            //

            PlaceInfo[] places = null;
            LevelInfo[] levels = null;
            bool result;
            try
            {
                result = proxy.GetBasicRouteInfo3(out places, out levels);
            }
            catch
            {
                result = false;
            }
            if (!result)
            {
                MessageBox.Show(
                    "経路およびクラス情報の読み込みに失敗しました。");
                return;
            }
            // 出発地のリストを作成
            deptCombo.DisplayMemberPath = "PlaceName";
            deptCombo.SelectedValuePath = "PlaceCode";
            deptCombo.ItemsSource = places;
            deptCombo.SelectedIndex = 0;
            // 到着地のリストを作成
            arrvCombo.DisplayMemberPath = "PlaceName";
            arrvCombo.SelectedValuePath = "PlaceCode";
            arrvCombo.ItemsSource = places;
            arrvCombo.SelectedIndex = places.Length > 1 ? 1 : 0;
            // クラス情報にワイルドカードを追加
            LevelInfo [] levelsExtend = new LevelInfo[levels.Length+1];
            Array.Copy(levels, 0, levelsExtend, 1, levels.Length);
            levelsExtend[0] = new LevelInfo() { LevelCode = "", LevelName = "全指定" };
            levelCombo.DisplayMemberPath = "LevelName";
            levelCombo.SelectedValuePath = "LevelCode";
            levelCombo.ItemsSource = levelsExtend;
            levelCombo.SelectedIndex = 0;

            // 日付をコンボボックスに設定する
            // (サンプルデータの期間にあわせて、初期値値を 2011/07/01～2011/07/01とします)
            fromDate.SelectedDate = new DateTime(2011, 7, 1);
            toDate.SelectedDate = new DateTime(2011, 7, 1);
        }

        // メッセージをクリアする
        private void ClearMessages()
        {
            QueryMessageLabel.Content = "";
            ReserveMessageLabel.Content = "";
        }

        // 開始日を指定した際、終了日が開始日よりも以前になったら終了日を修正する
        // 開始日と終了日のリストが同じ内容であることが前提
        private void fromDate_SelectedDateChanged(object sender, SelectionChangedEventArgs e)
        {
            if (fromDate.SelectedDate > toDate.SelectedDate)
                toDate.SelectedDate = fromDate.SelectedDate;
        }

        // 終了日を指定した際、終了日が開始日よりも以前になったら開始日を修正する
        // 開始日と終了日のリストが同じ内容であることが前提
        private void toDate_SelectedDateChanged(object sender, SelectionChangedEventArgs e)
        {
            if (fromDate.SelectedDate > toDate.SelectedDate)
                fromDate.SelectedDate = toDate.SelectedDate;
        }

        // [運行予定照会]ボタン
        private void QueryButton_Click(object sender, RoutedEventArgs e)
        {
            ClearMessages();
            QuerySchedule();
        }

        // [予約]ボタン
        private void ReserveButton_Click(object sender, RoutedEventArgs e)
        {
            ClearMessages();
            Reserve();
        }

        // 運行予定照会
        //   予約管理サービスから運行予定を取得してUIに設定する
        private void QuerySchedule()
        {
            var proxy = new ReservationClient();

            // 明示的な Windows ユーザーアカウントの指定
            //proxy.ClientCredentials.Windows.ClientCredential.UserName = "Administrator";
            //proxy.ClientCredentials.Windows.ClientCredential.Password = "p@ssw0rd";
            //

            try
            {
                bool result;
                ScheduleInfo[] schedules;
                string msg;
                result = proxy.QuerySchedule(
                    out schedules, out msg,
                    (DateTime)fromDate.SelectedDate,
                    (DateTime)toDate.SelectedDate + new TimeSpan(1, 0, 0, 0),
                    (string)deptCombo.SelectedValue, (string)arrvCombo.SelectedValue,
                    (string)levelCombo.SelectedValue);

                QueryMessageLabel.Content = msg;
            
                // 照会結果の運行予定をこのウィンドウに対してバインドする
                // ウィンドウ内の各コントロールは、{Binding 要素名} で特定の項目とバインドする
                this.DataContext = schedules;
            }
            catch (Exception ex)
            {
                // サービス自体の呼び出しに例外が発生した場合
                QueryMessageLabel.Content = ex.Message + "(" + ex.GetType().Name + ")";
            }

            // 以下はXAMLで設定済み
            //ScheduleDataGrid.AutoGenerateColumns = false;
            //ScheduleDataGrid.IsSynchronizedWithCurrentItem = true;
        }

        // 予約
        private void Reserve()
        {
            // 予約管理サービスへ予約時に追加するコメント(経路)
            const string frmReservationComment = " {0}‐{1} ";
            // 予約完了時のメッセージ
            const string frmReservationResult = " 予約番号:{0}";

            // 運行予定が選択されているか確認
            if (ScheduleDataGrid.SelectedIndex < 0)
            {
                ReserveMessageLabel.Content
                    = "予約を行うためには、運行予定一覧から予約対象の便を選択してください。";
                return;
            }

            // 必須項目の確認
            if (String.IsNullOrEmpty(LastNameTextBox.Text) ||
                String.IsNullOrEmpty(FirstNameTextBox.Text))
            {
                ReserveMessageLabel.Content
                    = "姓および名前は必須です。";
                return;
                // サービス側でも引数の検証を行っている
            }

            // 選択した運行データの参照
            ScheduleInfo sinfo = ScheduleDataGrid.SelectedItem as ScheduleInfo;
            System.Diagnostics.Debug.Assert(sinfo != null);

            // 予約時コメントの整形
            string cmtOnReserve = String.Format(
                frmReservationComment, sinfo.DeptName, sinfo.ArrvName);

            // 予約処理
            var proxy = new ReservationClient();
 
            bool result;
            string reservationCode;
            string msg;
            result = proxy.Reserve(
                out reservationCode, out msg,
                sinfo.ScheduleCode, sinfo.LevelCode,
                MemberCodeTextBox.Text, LastNameTextBox.Text, FirstNameTextBox.Text,
                sinfo.AppliedPrice, sinfo.AppliedTax, cmtOnReserve);

            if (result)
                ReserveMessageLabel.Content =
                    msg + String.Format(frmReservationResult, reservationCode);
            else
                ReserveMessageLabel.Content = msg;

            // 運行スケジュールの最新情報を再度照会
            // 再照会の前に選択していた行を再選択するために、選択すべき行を特定する
            // ただし、選択したのち、検索条件が変えられていた場合もあるので、
            // その場合は、新しい条件のもとに検索され、該当行がいなことがある
            QuerySchedule();
            var query = from s in ScheduleDataGrid.Items.Cast<ScheduleInfo>() 
                        where s.ScheduleCode == sinfo.ScheduleCode &&
                              s.LevelCode == sinfo.LevelCode
                        select s;
            try
            {
                var target = query.First();
                ScheduleDataGrid.SelectedItem = target;
            }
            catch
            {
            }
        }

    }
}
