﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

using Adwk.WpfApp.ServiceReference2;

namespace Adwk.WpfApp
{
    /// <summary>
    /// W02Confirm.xaml の相互作用ロジック
    /// </summary>
    public partial class W02Confirm : Window
    {

        // 予約確認/削除向けのプロキシ
        // ステートを管理するためメンバー変数として確保
        private ReservationClient proxy = null;

        public W02Confirm()
        {
            InitializeComponent();
        }

        // メッセージをクリアする
        private void ClearMessages()
        {
            ConfirmMessageLabel.Content = "";
            DeleteMessageLabel.Content = "";
        }

        // [予約確認]ボタン
        private void confirmButton_Click(object sender, RoutedEventArgs e)
        {
            ClearMessages();

            // プロキシが無ければ作成する
            if (proxy == null)
                proxy = new ReservationClient();

            bool result;
            ReservationInfo rinfo;
            string msg;

            result = proxy.QueryReservation1ByRCode(
                out rinfo, out msg,
                ReservationCodeTextBox.Text,
                LastNameTextBox.Text, FirstNameTextBox.Text);

            // 予約有無の問い合わせなので、予約がない場合でも
            // true (正常処理) が返る
            // 予約がない場合は null にしてデータバインディングを解除
            if (result && rinfo != null)
                this.DataContext = new ReservationInfo[1] { rinfo };
            else
                this.DataContext = null;

            ConfirmMessageLabel.Content = msg;
        }

        // [予約削除]ボタン
        private void DeleteButton_Click(object sender, RoutedEventArgs e)
        {
            ClearMessages();

            // 削除対象の選択有無の確認
            if (ReservationDataGrid.SelectedItem == null)
            {
                MessageBox.Show("削除すべき予約を1行だけ選択してください。");
                return;
            }

            // 削除データの特定
            ReservationInfo rinfo =
                ReservationDataGrid.SelectedItem as ReservationInfo;
            System.Diagnostics.Debug.Assert(rinfo != null);

            // 削除確認のメッセージ
            MessageBoxResult mbResult = MessageBox.Show(
                this, String.Format("予約 {0} を削除します。\n",
                          rinfo.ReservationCode) +
                          "一度削除すると、元には戻せません。\n" +
                          "予約削除を実行しますか。",
                "予約削除の確認", MessageBoxButton.YesNo);
            if (mbResult != MessageBoxResult.Yes)
            {
                return;
            }

            //同じサービスのステートを使用するため、
            //作成済みのプロキシを使用して予約サービスを呼び出す
            if (proxy == null)
                proxy = new ReservationClient();

            bool result;
            string msg;

            result = proxy.DeleteReservationByRCode(
                        out msg, rinfo.ReservationCode);

            DeleteMessageLabel.Content = msg;

            // 削除に成功した場合は、データバインディングは解除
            if(result)
                this.DataContext = null;
        }

        // プロキシを経由して明示的にサービス側のリソースを開放
        private void Window_Unloaded(object sender, RoutedEventArgs e)
        {
            if (proxy != null)
            {
                try
                {
                    proxy.Reset();
                    proxy.Close();
                }
                catch
                {
                    proxy.Abort();
                }
                proxy = null;
            }
        }
    }
}
