﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

using Adwk.WpfApp.ServiceReference1;
using Adwk.WpfApp.ServiceReference2;
using Adwk.WpfApp.ServiceReference3;

namespace Adwk.WpfApp
{
    /// <summary>
    /// W03CheckIn.xaml の相互作用ロジック
    /// </summary>
    public partial class W03CheckIn : Window
    {
        public W03CheckIn()
        {
            InitializeComponent();
        
            // ユーザーインターフェイスに必要な予約関連情報の取得と設定
            BuildUI();
        }

        // ユーザーインターフェイスに必要な予約関連情報の取得と設定
        private void BuildUI()
        {
            // 経路とクラス情報を取得してコンボボックスに設定する
            var proxy = new RBasicInfoClient();
            PlaceInfo[] places = null;
            LevelInfo[] levels = null;
            bool result;
            try
            {
                result = proxy.GetBasicRouteInfo3(out places, out levels);
            }
            catch
            {
                result = false;
            }
            if (!result)
            {
                MessageBox.Show(
                    "経路およびクラス情報の読み込みに失敗しました。");
                return;
            }
            // 出発地のリストを作成
            deptCombo.DisplayMemberPath = "PlaceName";
            deptCombo.SelectedValuePath = "PlaceCode";
            deptCombo.ItemsSource = places;
            deptCombo.SelectedIndex = 0;
            // 到着地のリストを作成
            arrvCombo.DisplayMemberPath = "PlaceName";
            arrvCombo.SelectedValuePath = "PlaceCode";
            arrvCombo.ItemsSource = places;
            arrvCombo.SelectedIndex = places.Length > 1 ? 1 : 0;

            // 日付をコンボボックスに設定する
            // (サンプルデータの期間にあわせて、初期値値を 2011/07/01とします)
            ThisDate.SelectedDate = new DateTime(2011, 7, 1);

            // 初期メッセージ
            QueryMessageLabel.Content = "乗船手続きを行う便を特定してください。";
        }

        // メッセージをクリアする
        private void ClearMessages()
        {
            CheckInMessageLabel.Content = "";
        }

        // [運行予定照会]ボタン
        private void QueryButton_Click(object sender, RoutedEventArgs e)
        {
            ClearMessages();
            QuerySchedule();
        }

        // 運行予定照会
        //   予約管理サービスから運行予定を取得してUIに設定する
        private void QuerySchedule()
        {
            var proxy = new ReservationClient();
            bool result;
            ScheduleInfo[] schedules;
            string msg;
            result = proxy.QuerySchedule(
                out schedules, out msg,
                (DateTime)ThisDate.SelectedDate,
                (DateTime)ThisDate.SelectedDate + new TimeSpan(1, 0, 0, 0),
                (string)deptCombo.SelectedValue, (string)arrvCombo.SelectedValue,
                (string)""); //クラス(レベル)はワイルドカード

            QueryMessageLabel.Content = msg;

            // 照会結果の運行予定をこのウィンドウに対してバインドする
            // ウィンドウ内の各コントロールは、{Binding 要素名} で特定の項目とバインドする
            this.DataContext = schedules;

            // 以下はXAMLで設定済み
            //ScheduleDataGrid.AutoGenerateColumns = false;
            //ScheduleDataGrid.IsSynchronizedWithCurrentItem = true;
        }

        // [乗船受付]ボタン
        private void CheckInButton_Click(object sender, RoutedEventArgs e)
        {
            ClearMessages();

            // 手続き完了時のメッセージ
            const string msgDone = "{0} ({1}様 予約コード:{2})";

            // 乗船対象の選択有無の確認
            if (ScheduleDataGrid.SelectedItem == null)
            {
                MessageBox.Show("乗船対象となる便を1行だけ選択してください。");
                return;
            }

            // 乗船データの特定
            ScheduleInfo sinfo =
                ScheduleDataGrid.SelectedItem as ScheduleInfo;
            System.Diagnostics.Debug.Assert(sinfo != null);
            
            // 乗船手続き用のサービスプロキシの作成
            var proxy = new RBoardingClient();

            // 明示的な Windows ユーザーアカウントの指定
            //proxy.ClientCredentials.Windows.ClientCredential.UserName = "administrator";
            //proxy.ClientCredentials.Windows.ClientCredential.Password = "p@ssw0rd";
            //

            string msgResult = "";

            try
            {
                bool result;
                string msg;

                // 乗船受付
                result = proxy.CheckIn(
                    out msg,
                    (string)sinfo.ScheduleCode,
                    (string)sinfo.LevelCode,
                    (string)ReservationCodeTextBox.Text,
                    (string)MemberCodeTextBox.Text,
                    (string)LastNameTextBox.Text,
                    (string)FirstNameTextBox.Text);

                // 手続き完了メッセージ表示と入力項目クリア
                if (result)
                {
                    // 手続き完了メッセージ作成
                    msgResult = String.Format(
                           msgDone, msg, LastNameTextBox.Text,
                           ReservationCodeTextBox.Text);
                    // 入力項目クリア
                    ReservationCodeTextBox.Text = "";
                    LastNameTextBox.Text = "";
                    FirstNameTextBox.Text = "";
                    MemberCodeTextBox.Text = "";
                }
                else
                {
                    // サービス自体の呼び出しには成功したが、処理に失敗した場合
                    msgResult = msg;
                }
            }
            catch (Exception ex)
            {
                // サービス自体の呼び出しに例外が発生した場合
                msgResult = ex.Message + "(" + ex.GetType().Name + ")";
            }

            CheckInMessageLabel.Content = msgResult;
        }
    }
}
